/*****************************************************************************
 *  Rohde&Schwarz SFC Compact Modulator instrument driver     
 *  VXIpnp Style Instrument Driver
 *  Original Release: 06/2011
 *  By: J.Kominek, P.Marcalik
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History: see ChangeLog
 *
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include "rssfc.h"
#include "rsidr_core.h"
#include "rssfc_attributes.h"

/*****************************************************************************
 *- Useful Macros & Constants -----------------------------------------------*
 *****************************************************************************/

#define RSSFC_SIMULATION_ID_QUERY      "Rohde&Schwarz,SFC,2115.3510k02/100145,02.80.00.8"  /* Default definition of ID Query Response for simulation */
#define RSSFC_VALID_ID_RESPONSE_STRING "Rohde&Schwarz,SFC"      /* Valid response for identification query */
#define RSSFC_SIMULATION_OPT_QUERY     "SFC-B1,SFC-B3,SFC-B4,SFC-B30"      /* Simulated response for *OPT? command */

#define RSSFC_IO_BUFFER_SIZE           1024L   /* I/O buffer size */
#define RSSFC_OPC_TIMEOUT              30000L   /* Maximum time to wait for OPC in milliseconds */
#define IEEE_488_2_ERROR_MASK          0x3C    /* 488.2 Event Status Register (ESR) Bits */
#define RSSFC_TOKEN_BUFFER_SIZE        50      /* Size for rep capability values */
#define MAX_OPC_TIME                   60000L 


/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
    
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/

static ViString  statusRegisterArr[] = {"COND","EVEN","ENAB","PTR","NTR"};
static ViString  layerArr[] = {"A","B","C"};
static ViString  cmmbPlchRsArr[] = {"0","1","2","3"};      
static ViString  cmmbPlchLdpdArr[] = {"R1_2","R3_4"};   
static ViString  cmmbPlchConstArr[] = {"C2","C4","C16"};
static ViString  dvbt2PlpArr[] = {"","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16"};
static ViString  dvbC2PlpArr[] = {"","1","2","3","4"};
static ViString  dSliceArr[] = {"1","2","3","4"}; 
static ViString  notchArr[] = {"1","2","3","4"};

/*****************************************************************************
 *- Utility Function Declarations (Non-Exported) ----------------------------*
 *****************************************************************************/

static ViStatus rssfc_InitAttributes (ViSession vi);
static ViStatus rssfc_DefaultInstrSetup (ViSession openInstrSession);
static ViStatus rssfc_CheckStatus (ViSession vi);
static ViStatus rssfc_WaitForOPC (ViSession vi, ViInt32 maxTime);
static ViStatus rssfc_ReadDataUnknownLength(ViSession instrSession,
                ViChar **outBuffer, ViUInt32 *count);



static ViStatus rssfc_GetActualRepCaps (ViSession instrSession, ViAttr attribute, 
                                        ViString repCaps, ViInt32 index,ViChar retVal[]);
    /*********************************************************
        Functions reserved for class driver use only.
        End-users should not call these functions.
     *********************************************************/

ViStatus _VI_FUNC  rssfc_RsClose (ViSession vi);


ViStatus _VI_FUNC rssfc_SetErrorInfo (ViSession session,
                          ViBoolean overWrite,
                          ViStatus primaryError,
                          ViStatus secondaryError,
                          ViConstString errorElaboration);

ViStatus _VI_FUNC rssfc_GetErrorInfo (ViSession session,
                          ViStatus *primaryError,
                          ViStatus *secondaryError,
                          ViChar errorElaboration[]);
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

ViStatus rssfc_CheckStatusCallback (ViSession io);
ViStatus rssfc_WaitForOPCCallback (ViSession io);

ViStatus rssfc_quotedString_WriteCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, void *value);
ViStatus rssfc_hex_WriteCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, void *value);
ViStatus rssfc_hex_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value);
/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rssfc_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rssfc_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rssfc_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rssfc_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == VI_NULL)
        {
        (void) Rs_SetErrorInfo (VI_NULL, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, VI_NULL, VI_NULL, &instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 20000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList, rssfc);
    checkErr( Rs_SpecificDriverNew (instrSession, "rssfc", newOptionString, attrList));

    /* Build repeated capability table */
    REPCAPTABLE(repCapTable, rssfc);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rssfc_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rssfc_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSSFC_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, (ViAttr) RSSFC_ATTR_ID_QUERY_RESPONSE,
                                   0, RSSFC_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSSFC_VALID_ID_RESPONSE_STRING, strlen(RSSFC_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rssfc_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        ViSession   rmSess = VI_NULL;

        (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSess), &rmSess);
        
        if (instrSession != 0)
            (void) viClose (instrSession);
        (void) viClose (rmSess);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    if (error < VI_SUCCESS)
        (void) Rs_Dispose (*newInstrSession);

    return error;
}

/*****************************************************************************
 * Function: rssfc_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    // checkErr( Rs_LockSession (vi, VI_NULL));
    checkErr( rssfc_RsClose (instrSession));

Error:

    // Rs_UnlockSession (vi, VI_NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/*****************************************************************************
 * Function: rssfc_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rssfc_close or by an RS class driver.
 *
 *           Note:  This function must close the I/O session and set
 *           RS_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_RsClose (ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    ViSession   rmSession = VI_NULL;

    /* Do not lock here. The caller manages the lock. */

    (void) Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_IO_SESSION, 0, VI_NULL);
    (void) Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViInt32) sizeof (rmSession), &rmSession);
        
    (void) viClose (instrSession);
    if (rmSession != 0)
        (void) viClose (rmSession);
   
    return error;
}


ViStatus _VI_FUNC rssfc_SetErrorInfo (ViSession session,
                          ViBoolean overWrite,
                          ViStatus primaryError,
                          ViStatus secondaryError,
                          ViConstString errorElaboration)
{
    return Rs_SetErrorInfo (session,
                          overWrite,
                          primaryError,
                          secondaryError,
                          errorElaboration);
}

ViStatus _VI_FUNC rssfc_GetErrorInfo (ViSession session,
                          ViStatus *primaryError,
                          ViStatus *secondaryError,
                          ViChar errorElaboration[])
{
    return Rs_GetErrorInfo (session,
                          primaryError,
                          secondaryError,
                          errorElaboration);
}



/*****************************************************************************
 * Function: rssfc_reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViUInt32    tout    = 0;
    ViBoolean   opc     = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
            viCheckErr (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &tout));
            viCheckErr (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 90000));
        
            viCheckErr (Rs_WriteInstrData (instrSession, "*RST;*OPC?"));
            viCheckErr (viScanf (instrSession, "%hu", &opc));
            
            viCheckErr (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, tout));
        }

    checkErr( rssfc_DefaultInstrSetup (instrSession));
    checkErr( rssfc_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, VI_NULL);

    return error;
}

/*****************************************************************************
 * Function: rssfc_ResetWithDefaults
 * Purpose:  This function resets the instrument and applies default settings
 *           from the RS Configuration Store based on the logical name
 *           from which the session was created.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_ResetWithDefaults (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    checkErr( rssfc_reset(instrSession));
    checkErr( Rs_ApplyDefaultSetup(instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/****************************************************************************
 *  Function: rssfc_Disable
 *  Purpose:  This function places the instrument in a quiescent state as
 *            quickly as possible.
 ****************************************************************************/
ViStatus _VI_FUNC rssfc_Disable (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
	instrSession;
    
/*    checkErr( Rs_LockSession (vi, VI_NULL));

    if (Rs_Simulating(vi) == 0)
        {
        }
        */
//Error:
//    (void) Rs_UnlockSession (vi, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfc_self_test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (testResult == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result")
    if (testMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?"));
        viCheckErr( viScanf (instrSession, "%hd,\"%256[^\"]", testResult, testMessage));

        }
    else
        {
        /* Simulate Self Test */
        if (testResult != NULL)
            *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rssfc_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfc_revision_query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (driverRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFC_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSSFC_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rssfc_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfc_error_query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_error_query (ViSession instrSession, ViInt32 *errCode,
                                        ViChar errMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [256] = "SYST:ERR?\n";

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (errCode == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
    {
        viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen (buffer), NULL));
        viCheckErr (viRead (instrSession, (ViBuf)buffer, 255, NULL));
        sscanf (buffer, "%ld,\"%255[^\"]", errCode, errMessage);
    }
    else
    {
            /* Simulate Error Query */
        if (errCode  != NULL)
            *errCode = 0;
        strcpy (errMessage, "No error.");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: rssfc_error_message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass VI_NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
 
            RSSFC_ERROR_CODES_AND_MSGS,
            {VI_NULL,                               VI_NULL}
        };

    (void) Rs_LockSession(instrSession, VI_NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set OPC Timeout                                                 */
/* Purpose:  Sets new OPC timeout                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetOPCTimeout(ViSession instrSession, ViInt32 timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_SetAttribute (instrSession, VI_NULL, RSSFC_ATTR_OPC_TIMEOUT, 0, &timeout));  
    
Error:    
    return error;
}

/*===========================================================================*/
/* Function: Get OPC Timeout                                                 */
/* Purpose:  Gets new OPC timeout                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_GetOPCTimeout(ViSession instrSession, ViInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFC_ATTR_OPC_TIMEOUT, 0, sizeof(timeout), timeout));  
    
Error:    
    return error;
}
 
/*****************************************************************************
 * Function: rssfc_GetError and rssfc_ClearError Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
        {
        if (errorCode == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
            }
        if (description == VI_NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }

    else
        {
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rssfc_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rssfc_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (elaboration[0] != '\0')
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) strlen(errorMessage) + 1;

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        if (description != NULL)
            description[actualSize-1] = '\0';
        }

    if (errorCode != 0)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rssfc_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/*****************************************************************************
 * Function: rssfc_LockSession and rssfc_UnlockSession Functions
 * Purpose:  These functions enable the instrument driver user to lock the
 *           session around a sequence of driver calls during which other
 *           execution threads must not disturb the instrument state.
 *
 *           NOTE:  The callerHasLock parameter must be a local variable
 *           initialized to VI_FALSE and passed by reference, or you can pass
 *           VI_NULL.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_LockSession (ViSession instrSession, ViBoolean *callerHasLock)
{
    return Rs_LockSession(instrSession, callerHasLock);
}

ViStatus _VI_FUNC rssfc_UnlockSession (ViSession instrSession, ViBoolean *callerHasLock)
{
    return Rs_UnlockSession(instrSession,callerHasLock);
}

/*****************************************************************************
 *------------ User-Callable Functions (Exportable Functions) ---------------*
 *****************************************************************************/

/*****************************************************************************
 *------------------------- TX Application ----------------------------------*
 *****************************************************************************/
/*****************************************************************************
 *---------------------------- Frequency ------------------------------------*
 *****************************************************************************/

/*===========================================================================*
 * Function: Configure Frequency
 * Purpose:  This function configures Tx Frequency.
 *===========================================================================*/
ViStatus _VI_FUNC rssfc_confTxFreq(ViSession instrSession,
                                   ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm( rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_FREQ_CW, frequency), 2, "Frequency");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set Tx Actual Center Frequency
 * Purpose:  This function sets the RF output frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfc_SetTxActualCenterFrequency (ViSession instrSession,
                                                    ViReal64 RFOutputFrequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_FREQ_ACT_CENTER, RFOutputFrequency),
                                            2, "RF Output Frequency");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    


/*============================================================================/
 * Function: Set Tx Frequency Channel
 * Purpose:  This function sets the channel and RF output frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfc_SetTxFrequencyChannel (ViSession instrSession,
                                               ViInt32 frequencyChannel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_FREQ_CHANNEL, frequencyChannel),
                                            2, "Frequency Channel");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    
                                               


/*============================================================================/
 * Function: Frequency Channel Table Operations
 * Purpose:  This function selects frequency channel table operations.
/============================================================================*/
ViStatus _VI_FUNC rssfc_FrequencyChannelTableOperations (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString channelName)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    switch (operation){
        case RSSFC_VAL_FREQ_CHAN_TAB_SELECT:
             viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_FREQ_CHANNEL_TABLE_SELECT, channelName),
                         3, "Channel Name"); 
        break;
        
        case RSSFC_VAL_FREQ_CHAN_TAB_STORE:
             viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_FREQ_CHANNEL_TABLE_STORE, channelName),
                         3, "Channel Name"); 
        break;
        
        case RSSFC_VAL_FREQ_CHAN_TAB_DELETE:
             viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_FREQ_CHANNEL_TABLE_DELETE, channelName),
                         3, "Channel Name"); 
        break;
        }
    }
 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                         


/*============================================================================/
 * Function: Configure Channel Table Frequency
 * Purpose:  This function configures channel table frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureChannelTableFrequency (ViSession instrSession,
                                                        ViInt32 channel,
                                                        ViString channelName,
                                                        ViReal64 channelFrequency)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if ((channel < 1) || (channel > 100))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Channel");
    if ((channelFrequency < 300.0e3) || (channelFrequency > 3.0e9))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Channel Frequency");         
    
    if (Rs_Simulating(instrSession) == 0){     
    
        viCheckErr( viPrintf (instrSession, ":FREQ:CHAN:TABL:NAME %ld,\"%s\"", channel, channelName)); 
        
        viCheckErr( viPrintf (instrSession, ":FREQ:CHAN:TABL:FREQ %ld,%lf", channel, channelFrequency));   
    }

    viCheckErr(rssfc_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
       

/*============================================================================/
 * Function: Set Frequency VSB Reference Point
 * Purpose:  This function determines the reference point for the frequency
 *           setting in the 8VSB modulation type.
/============================================================================*/
ViStatus _VI_FUNC rssfc_SetFrequencyVSBReferencePoint (ViSession instrSession,
                                                       ViInt32 frequencyRefPoint)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm( rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_FREQ_VSB_REF_POINT, frequencyRefPoint),
                                            2, "Frequency Reference Point");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    

/*============================================================================/
 * Function: Read TX Frequency
 * Purpose:  This function queries the RF output frequency.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readTxFreq (ViSession instrSession, ViReal64 *frequency)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!frequency) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "TX Frequency");
    viCheckErr (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_FREQ_CW,
                                            frequency));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*****************************************************************************
 *---------------------------- Level ----------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Level
 * Purpose:  This function configures Tx Level.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTxLevel (ViSession instrSession, ViReal64 level,
                                     ViBoolean rf, ViReal64 userLimit,
                                     ViInt32 attenuatorMode, ViInt32 levelUnit)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_POW_LEVEL, level),
                                2, "Level"); 
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_RFOUT_STATE, rf),
                                3, "RF"); 
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_POW_USER_LIMIT, userLimit),
                                4, "User Limit"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_RFOUT_AMODE, attenuatorMode),
                                5, "Attenuator Mode"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_POW_LEV_UNIT, levelUnit),
                                6, "Level Unit");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TX Level
 * Purpose:  This function queries the RF output level in CW mode.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readTxLevel (ViSession instrSession, ViReal64 *level)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!level) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "TX Level");
    viCheckErr (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_POW_LEVEL,
                                            level));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Get Att Fixed Range
 * Purpose:  This function queries the minimum and maximum level that can be 
 *           set without altering the attanuator in the mode Attenuator 
 *           Fixed.
/============================================================================*/
ViStatus _VI_FUNC rssfc_getTxAttFixedRange (ViSession instrSession,
                                            ViReal64 *lower, ViReal64 *upper)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_RFOUT_ATT_MIN, lower),
                                2, "Lower");    
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_RFOUT_ATT_MAX, upper),
                                3, "Upper");                                    
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Peak Envelope Power
 * Purpose:  This function returns the peak envelope power(PEP) in dBm for
 *           digital modulation and all digital standards.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readPeakEnvelopePower (ViSession instrSession,
                                               ViReal64 *peakEnvelopePower)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!peakEnvelopePower) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Peak Envelope Power");
    viCheckErr (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_POW_PEAK_ENVELOPE_POWER,
                                            peakEnvelopePower));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *---------------------------- Modulation -----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Modulation
 * Purpose:  This function configures Tx modulator.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTxModulation (ViSession instrSession,
                                          ViBoolean modulation,
                                          ViInt32 signalSource,
                                          ViInt32 transmissionStandard,
                                          ViInt32 spectrum)
                                         
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MOD_DM_SIGNAL_SOURCE, signalSource),
                                3, "Signal Source"); 
    
    if(signalSource == RSSFC_VAL_DM_SOURCE_DTV)
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MOD_DM_TRANSMISSION_STANDARD, transmissionStandard),
                                    4, "Transmission Standart");
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MOD_DM_SPECTRUM, spectrum),
                                5, "Spectrum");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MOD_DM_MODULATION, modulation),
                                2, "Modulation"); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Modulation ATV Transmission Standard
 * Purpose:  This function configures the transmission standard.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAtvTransStandard (ViSession instrSession,
                                         ViInt32 ATVTransmissionStandard)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MOD_DM_ATV_TRANSMISSION_STANDARD,
                            ATVTransmissionStandard), 2, "ATV Transmission Standard");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Modulation Settings
 * Purpose:  This function configures Tx modulator settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTxModulationSettings(ViSession instrSession,
                                                 ViInt32 iqDigitalInMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MOD_DIGITAL_IN_MODE,
                                            iqDigitalInMode), 2, "I/Q Digital In Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Audio Standard
 * Purpose:  This function configures the audio standard.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAudioStandard(ViSession instrSession,
                                          ViInt32 audioStandard)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MOD_DM_AUDIO_STANDARD,
                                            audioStandard), 2, "Audio Standard");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Set Modulation IQ Swap
 * Purpose:  This function sets the IQ swap state.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_setModulationIQSwap (ViSession instrSession,
                                                 ViBoolean iqSwap)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MOD_DM_IQ_SWAP, iqSwap),
                         2, "IQ Swap");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Modulation Signal Info and Statistics
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readModulationSigInfoStat (ViSession instrSession,
                                                   ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_MOD_DIG_IO_INP_RMS) || (value > RSSFC_VAL_MOD_DIG_IO_INP_PEAK))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value) {
        case RSSFC_VAL_MOD_DIG_IO_INP_RMS:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MOD_DIGITAL_IQ_INPUT_RMS_VAL,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MOD_DIG_IO_INP_PEAK:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MOD_DIGITAL_IQ_INPUT_PEAK_VAL,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read Digital IQ Symbol Rate
/* Purpose:  This function queries the symbolrate present at the digital I/Q 
/*           Input.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readDigitalIqSymbolRate(
    ViSession   instrSession,
    ViReal64*   iqSymbolRate
)
{
       ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
     
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "",
        RSSFC_ATTR_MODULATION_DIGITAL_IQ_INPUT_SYMBOL_RATE, iqSymbolRate), 2, "IQ Symbol Rate");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*============================================================================/
 * Function: Modulation IQ Input Refresh
 * Purpose:  This function activates the recalculation of the output power
 *           according to the level measured at the I/Q DIGITAL or I/Q ANALOG
 *           input.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTxModIqInputRefresh (ViSession instrSession,
                                                 ViInt32 refresh)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((refresh < RSSFC_VAL_MOD_IQ_INPUT_DIG) || (refresh > RSSFC_VAL_MOD_IQ_INPUT_DIG))
        
    viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Refresh");

            viCheckErr (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_MOD_DIGITAL_IQ_INPUT_REFRESH, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*****************************************************************************
 *------------------------------ Digital TV ---------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *-------------------------------- DVB-T/H ----------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *------------------------------------------ Input Signal -------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure DVBT Input Signal
 * Purpose:  This function configures DVBT Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTInputSignal (ViSession instrSession,
                                             ViInt32 priorityPath,
                                             ViInt32 source, ViInt32 input,
                                             ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFC_VAL_PRIORITY_HIGH) || (priorityPath > RSSFC_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
        
    switch (priorityPath){
        case RSSFC_VAL_PRIORITY_HIGH:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_SOUR, source),
                         3, "Source"); 
			if (source == RSSFC_IQC_SIG_SOUR_EXT)
            	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_INP, input),
                         4, "Input");
			if (source != RSSFC_IQC_SIG_SOUR_TEST)
            	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_STUF, stuffing),
                         5, "Stuffing");                         
        break;
        case RSSFC_VAL_PRIORITY_LOW:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_SOUR_LOW, source),
                         3, "Source");
			if (source == RSSFC_IQC_SIG_SOUR_EXT) 
            	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_INP_LOW, input),
                         4, "Input");
			if (source != RSSFC_IQC_SIG_SOUR_TEST)
            	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_STUF_LOW, stuffing),
                         5, "Stuffing");                         
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Test Signal
 * Purpose:  This function configures DVBT Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTTestSignal(ViSession   instrSession,
                                           ViInt32 priorityPath,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFC_VAL_PRIORITY_HIGH) || (priorityPath > RSSFC_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    
    switch (priorityPath){
        case RSSFC_VAL_PRIORITY_HIGH:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_TEST, testSignal),
                         3, "Test Signal"); 
        break;
        case RSSFC_VAL_PRIORITY_LOW:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_TEST_LOW, testSignal),
                         3, "Test Signal"); 
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBTValues(ViSession   instrSession,
                                       ViInt32 priorityPath,
                                       ViInt32 value,
                                       ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFC_VAL_PRIORITY_HIGH) || (priorityPath > RSSFC_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    if ((value < RSSFC_VAL_PACKET_LENGTH) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value");        

    switch (priorityPath){
        case RSSFC_VAL_PRIORITY_HIGH:
            switch (value){
                case RSSFC_VAL_PACKET_LENGTH:
                    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_PACKET_LENGTH,
                                                            &int_result), 4, "Result"); 
                    *result = (ViReal64)int_result;
                break;
                
                case RSSFC_VAL_MAX_USEFUL_DRATE:
                    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBT_MAX_USEFUL_DATA_RATE,
                                                             result), 4, "Result");
                break;
                case RSSFC_VAL_MEAS_DRATE:
                    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBT_MEASURED_DATA_RATE,
                                                             result), 4, "Result");
                break;
                case RSSFC_VAL_USEFUL_DRATE:
                    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBT_USEFUL_DATA_RATE,
                                                             result), 4, "Result");
                break;
					
             }
            break;
        case RSSFC_VAL_PRIORITY_LOW:
            switch (value){
                case RSSFC_VAL_PACKET_LENGTH:
                    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_PACKET_LENGTH_LOW,
                                                            &int_result), 4, "Result"); 
                    *result = (ViReal64)int_result;
                break;
                
                case RSSFC_VAL_MAX_USEFUL_DRATE:
                    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBT_MAX_USEFUL_DATA_RATE_LOW,
                                                             result), 4, "Result");
                break;
                case RSSFC_VAL_MEAS_DRATE:
                    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBT_MEASURED_DATA_RATE_LOW,
                                                             result), 4, "Result");
                break;
                case RSSFC_VAL_USEFUL_DRATE:
                    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBT_USEFUL_DATA_RATE_LOW,
                                                             result), 4, "Result");
                break;
				
             }
            break;
                
    }    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Coding
 * Purpose:  This function configures DVBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTCoding (ViSession instrSession,
                                        ViInt32 channelBandwidth,
                                        ViInt32 fftMode, ViInt32 guardInterval,
                                        ViInt32 hierarchy, ViInt32 constellation)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_CHAN, channelBandwidth),
                         2, "Channel Bandwidth"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_FFT_MODE, fftMode),
                         4, "FFT Mode"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_GUAR_INT, guardInterval),
                         5, "Guard Interval"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_CONS, constellation),
                         7, "Constellation");                          
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_HIER, hierarchy),
                         6, "Hierarchy");                          

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Code Rate
 * Purpose:  This function configures DVBT code rate.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTCodeRate(ViSession instrSession,
                                         ViInt32 priorityPath,
                                         ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((priorityPath < RSSFC_VAL_PRIORITY_HIGH) || (priorityPath > RSSFC_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
    
    switch (priorityPath){
        case RSSFC_VAL_PRIORITY_HIGH:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_RATE, codeRate),
                         3, "Code Rate"); 
        break;
        case RSSFC_VAL_PRIORITY_LOW:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_RATE_LOW, codeRate),
                         3, "Code Rate"); 
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBH Coding
 * Purpose:  This function configures DVB-H coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBHCoding(ViSession instrSession,
                                       ViBoolean state,
                                       ViInt32 symbolInterleaverMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_DVHS, state),
                         2, "State");
	if (state == VI_TRUE)
   		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_DVBH_SYMB, symbolInterleaverMode),
                         3, "Symbol Interleaver Mode");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBT Special Settings
 * Purpose:  This function enables use of special DVBT coding parameters - 
 *           part 1.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableDVBTSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Special Settings
 * Purpose:  This function configures special DVBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTSpecial(ViSession   instrSession,
                                        ViInt32 priorityPath,
                                        ViBoolean   reedSolomon)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if ((priorityPath < RSSFC_VAL_PRIORITY_HIGH) || (priorityPath > RSSFC_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
        
    switch (priorityPath){
        case RSSFC_VAL_PRIORITY_HIGH:

            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_REED, reedSolomon),
                         3, "Reed Solomon");   
        break;
        case RSSFC_VAL_PRIORITY_LOW:
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_REED_LOW, reedSolomon),
                         3, "Reed Solomon");   
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Packets and Payload
 * Purpose:  This function configures DVBT coding settings - packets.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTPacketsPayload(ViSession   instrSession,
                                               ViInt32 testTSPacket,
                                               ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_TSP, testTSPacket),
                         2, "Test TS Packet");

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_PRBS, prbs),
                         3, "PRBS");     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the DVBT signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_ConfigureDVBTPayloadTestStuff (ViSession instrSession,
                                          ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 2, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT TPS
 * Purpose:  This function configures DVBT coding TPS settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTTPS(ViSession instrSession,
                                    ViInt32 tpsCellID,
                                    ViBoolean tpsResevedState,
                                    ViInt32 tpsReservedValue)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_CELL_ID, tpsCellID),
                         2, "TPS Cell ID");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_TPSR_STAT,
                                              tpsResevedState), 3, "TPS Reseved State"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBT_TPSR_VAL,
                                            tpsReservedValue), 4, "TPS Reserved Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT Timeslicing and MPE
 * Purpose:  This function configures DVBT coding Time Slicing and MPE.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBTTimeMPE(ViSession instrSession,
                                        ViInt32 priorityPath,
                                        ViBoolean timeSlicing,
                                        ViBoolean mpeFEC)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((priorityPath < RSSFC_VAL_PRIORITY_HIGH) || (priorityPath > RSSFC_VAL_PRIORITY_LOW))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Priority Path");
        
    switch (priorityPath){
        case RSSFC_VAL_PRIORITY_HIGH:
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_TIM, timeSlicing),
                         3, "Time Slicing"); 
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_MPEF, mpeFEC),
                         4, "MPE FEC");
        break;
        case RSSFC_VAL_PRIORITY_LOW:
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_TIM_LOW, timeSlicing),
                         3, "Time Slicing"); 
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBT_MPEF_LOW, mpeFEC),
                         4, "MPE FEC");
        break;
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}




/*****************************************************************************
 *----------------------------------- DTMB ---------------------------------*
 *****************************************************************************/


/*============================================================================/
 * Function: Configure DTMB Input Signal
 * Purpose:  This function configures DTMB Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDTMBInputSignal (ViSession instrSession,
                                             ViInt32 source, ViInt32 input,
                                             ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_SOUR, source),
                         2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT) 
    	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_INP, input),
                         3, "Input");
    if (source != RSSFC_IQC_SIG_SOUR_TEST)
	    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DTMB_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Test Signal
 * Purpose:  This function configures DTMB Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDTMBTestSignal(ViSession instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DTMB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDTMBValues(ViSession instrSession,
                                       ViInt32 value,
                                       ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_PACKET_LENGTH) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_PACKET_LENGTH:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_PACKET_LENGTH,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;
        break;
    
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DTMB_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DTMB_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DTMB_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Network
 * Purpose:  This function configures DTMB Network parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDTMBNetworkMode (ViSession instrSession,
                                             ViInt32 networkMode,
                                             ViBoolean singleCarrierMode,
                                             ViBoolean controlFrames,
                                             ViBoolean dualPilotTone)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32(instrSession, "", RSSFC_ATTR_IQC_DTMB_NETW,
                                                     networkMode), 2, "Network Mode");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DTMB_CODING_SINGLE_CARRIER_MODE, singleCarrierMode),
                         3, "Single Carrier Mode"); 
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DTMB_CODING_CONTROL_FRAMES, controlFrames),
                         4, "Control Frames");
    if (singleCarrierMode != 0)
    {
        viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DTMB_CODING_DUAL_PILOT_TONE, dualPilotTone),
                             5, "Dual Pilot Tone");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure DTMB Control
 * Purpose:  This function configures DTMB coding control.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDTMBControl(ViSession instrSession,
                                        ViInt32 channelBandwidth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_CHAN,
                                                     channelBandwidth), 2, "Channel Bandwidth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Coding
 * Purpose:  This function configures DTMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDTMBCoding (ViSession instrSession,
                                        ViInt32 constellation, ViInt32 codeRate,
                                        ViInt32 guardInterval,
                                        ViInt32 guardIntervalPN,
                                        ViInt32 timeInterleaver)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_CONS, constellation),
                         2, "Constellation"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_RATE, codeRate),
                         3, "Code Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_GUARD_INTERVAL, guardInterval),
                         4, "Guard Interval"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_SPECIAL_GUARD_INTERVAL_PN, guardIntervalPN),
                         5, "Guard Interval PN");       
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_TIME_INTERLEAVER, timeInterleaver),
                         6, "Time Interleaver");       
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure DTMB Settings
 * Purpose:  This function configures DTMB coding settings.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_confDTMBSettings (ViSession instrSession,
                                          ViInt32 testTSPacket,
                                          ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_TSP, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DTMB_PRBS, PRBS),
                         3, "PRBS");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DTMB Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the DTMB signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_ConfigureDTMBPayloadTestStuff (ViSession instrSession,
													ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
  	               
  	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DTMB_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 3, "Payload Test");   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- DVB-C ---------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure DVBC Input Signal
 * Purpose:  This function configures DVBC Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBCInputSignal(ViSession instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_SOUR, source),
                         2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT)
	    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_INP, input),
                         3, "Input");
    if (source != RSSFC_IQC_SIG_SOUR_TEST)	
    	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBC_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Test Signal
 * Purpose:  This function configures DVBC Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBCTestSignal(ViSession instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBC Packed Length
 * Purpose:  This function reads DVBC packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBCPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBC Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBCValues(ViSession instrSession,
                                       ViInt32 value,
                                       ViReal64* result
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBC_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBC_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBC_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Coding
 * Purpose:  This function configures DVBC coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBCCoding(ViSession instrSession,
                                       ViReal64 symbolRate,
                                       ViInt32 constellation,
                                       ViInt32 rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBC_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_CONS, constellation),
                         3, "Constellation");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_ROLL, rollOff),
                         4, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBC Special Settings
 * Purpose:  This function enables use of special DVBC coding parameters - 
 *           part 1.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableDVBCSpecial(ViSession   instrSession,
                                             ViBoolean   state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBC_SETT, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Special Reed Solomon
 * Purpose:  This function sets the Reed-Solomon coder on or off.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBCSpecialReedSolomon(ViSession   instrSession,
                                                      ViBoolean   reedSolomon)
{
    ViStatus    error = VI_SUCCESS;

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBC_REED, reedSolomon),
                 2, "Reed Solomon");   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Settings
 * Purpose:  This function configures DVBC coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBCSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_TSP, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBC_PRBS, prbs),
                         3, "PRBS");      
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBC Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the DVBC signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_ConfigureDVBCPayloadTestStuff (ViSession instrSession,
                                          ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 2, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- DVB-S ---------------------------------*
 *****************************************************************************/


/*============================================================================/
 * Function: Configure DVBS Input Signal
 * Purpose:  This function configures DVBS Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBSInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_SOUR, source),
                         2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT) 
		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_INP, input),
                         3, "Input");
	if (source != RSSFC_IQC_SIG_SOUR_TEST)
		viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBS_STUF, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Test Signal
 * Purpose:  This function configures DVBS Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBSTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBS Packed Length
 * Purpose:  This function reads DVBS packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBSPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBS Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBSValues(ViSession   instrSession,
                                       ViInt32 value,
                                       ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBS_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBS_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBS_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Coding
 * Purpose:  This function configures DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBSCoding(ViSession instrSession,
                                       ViReal64 symbolRate,
                                       ViInt32 constellation,
                                       ViInt32 rollOff,
                                       ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DVBS_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_CONS, constellation),
                         3, "Constellation");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_ROLL, rollOff),
                         4, "Roll Off");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_RATE, codeRate),
                         5, "Code Rate");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable DVBS Special Settings
 * Purpose:  This function enables use of special DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableDVBSSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBS_SETT_STATE, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Special Settings
 * Purpose:  This function configures special DVBS coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBSSpecialReedSolomon(ViSession instrSession,
                                        ViBoolean reedSolomon)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_DVBS_REED, reedSolomon),
                 2, "Reed Solomon");   
 Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS Settings
 * Purpose:  This function configures DVBS coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBSSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_TSP, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DVBS_PRBS, prbs),
                         3, "PRBS");      
  Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure DVBS Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the DVBS signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_ConfigureDVBSPayloadTestStuff (ViSession instrSession,
                                          ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 2, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *----------------------------------- DVB-S2 --------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure DVBS2 Input Signal
 * Purpose:  This function configures DVBS2 Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBS2InputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_INPUT_SIGNAL_SOURCE, source),
                         2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT)  	
    	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_INPUT_SIGNAL_INPUT, input),
                         3, "Input");
    if (source != RSSFC_IQC_SIG_SOUR_TEST)	
    	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBS2_INPUT_SIGNAL_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure DVBS2 Test Signal
 * Purpose:  This function configures DVBS2 Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBS2TestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBS2 Packed Length
 * Purpose:  This function reads DVBS2 packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBS2PackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_READ_DVBS2_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBS2 Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDVBS2Values(ViSession   instrSession,
                                       ViInt32 value,
                                       ViReal64*   result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_DVBS2_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_DVBS2_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_DVBS2_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBS2 Coding
 * Purpose:  This function configures DVBS2 coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBS2Coding (ViSession instrSession,
                                         ViReal64 symbolRate, ViInt32 constellation,
                                         ViInt32 FECFrame, ViBoolean pilots,
                                         ViInt32 rollOff, ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBS2_CODING_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_CODING_CONSTELLATION, constellation),
                         3, "Constellation");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_CODING_FEC_FRAME,FECFrame),
                         4, "FECFrame");                       
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBS2_CODING_PILOTS, pilots),
                         5, "Pilots");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_CODING_ROLL_OFF, rollOff),
                         6, "Roll Off");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_CODING_CODE_RATE, codeRate),
                         7, "Code Rate");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure DVBS2 Settings
 * Purpose:  This function configures DVBS2 coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDVBS2Settings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_SETTINGS_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_SETTINGS_PRBS, prbs),
                         3, "PRBS");      
  Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Configure DVBS2 Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the DVBS2 signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_confDVBS2PayloadTestStuff (ViSession instrSession,
                                          ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBS2_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 2, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*****************************************************************************
 *----------------------------------- 8VSB ----------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure 8VSB Input Signal
 * Purpose:  This function configures 8VSB Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_conf8VSBInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_8VSB_SOURCE, source),
                         2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT) 	
    	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_8VSB_INPUT_SIGNAL, input),
                         3, "Input");
    if (source != RSSFC_IQC_SIG_SOUR_TEST)	
    	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_8VSB_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
     
/*============================================================================/
 * Function: Configure 8VSB Test Signal
 * Purpose:  This function configures 8VSB Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_conf8VSBTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_8VSB_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read 8VSB Packed Length
 * Purpose:  This function reads 8VSB packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_read8VSBPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_8VSB_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Read 8VSB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_read8VSBValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_8VSB_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_8VSB_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_8VSB_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
       
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Set 8VSB Symbol Rate
 * Purpose:  This function allows setting of the symbol rate in the range
 *           10.762 MS/s +/-5%.
/============================================================================*/
ViStatus _VI_FUNC rssfc_set8VSBSymbolRate (ViSession instrSession,
                                           ViReal64 symbolRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_8VSB_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Read 8VSB Constellation
 * Purpose:  This function queries the VSB constellation firmly set to 8VSB.
/============================================================================*/
ViStatus _VI_FUNC rssfc_read8VSBConstellation (ViSession instrSession,
                                                 ViChar constellation[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE]="\0";  
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (constellation == NULL) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Constellation");
    viCheckErr (rssfc_GetAttributeViString (instrSession, "", RSSFC_ATTR_IQC_8VSB_CONSTELLATION,
                                            RSSFC_IO_BUFFER_SIZE, buffer));
    strncpy (constellation, buffer, strlen(buffer)); 
    if (constellation != NULL)
        constellation[strlen(buffer)] = '\0';
   
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Read 8VSB Roll Off
 * Purpose:  This function queries the rolloff, which is always 0.115 for  VSB.
/============================================================================*/
ViStatus _VI_FUNC rssfc_read8VSBRollOff (ViSession instrSession,
                                         ViReal64* rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_8VSB_ROLL_OFF,
                                                     rollOff), 2, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 


/*============================================================================/
 * Function: Configure 8VSB Settings
 * Purpose:  This function configures 8VSB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_conf8VSBSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_8VSB_TEST_TS_PACKED, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_8VSB_PRBS, prbs),
                         3, "PRBS");      
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure 8VSB Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the 8VSB signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_Configure8VSBPayloadTestStuff (ViSession instrSession,
                                          ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_8VSB_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 2, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*****************************************************************************
 *----------------------------------- J83B ----------------------------------*
 *****************************************************************************/
 
/*============================================================================/
 * Function: Configure J83B Input Signal
 * Purpose:  This function configures J83B Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confJ83BInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_SOURCE,source),
                         2, "Source"); 
	if (source == RSSFC_IQC_SIG_SOUR_EXT)
		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_INPUT_SIGNAL, input),
                         3, "Input");
	if (source != RSSFC_IQC_SIG_SOUR_TEST)
		viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_J83B_STUFFING, (ViBoolean)stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
     
/*============================================================================/
 * Function: Configure J83B Test Signal
 * Purpose:  This function configures J83B Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confJ83BTestSignal(ViSession   instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_TEST, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read J83B Packed Length
 * Purpose:  This function reads J83B packed length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readJ83BPackedLength (ViSession instrSession,
                                              ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Read J83B Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readJ83BValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_J83B_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_J83B_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_J83B_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
       
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Configure J83B Coding
 * Purpose:  This function configures J83B coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confJ83BCoding (ViSession instrSession,
                                        ViReal64 symbolRate,
                                        ViInt32 constellation,
                                        ViInt32 interleaver)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_CONSTELLATION, constellation),
                         3, "Constellation");
	viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_J83B_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_INTERLEAVER_MODE, interleaver),
                         4, "Interleaver")
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read J83B Roll Off
 * Purpose:  This function queries the rolloff, which is always 0.18 (64QAM)
 *           or 0.12 (256QAM) for J.83B.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readJ83BRollOff (ViSession instrSession,
                                         ViReal64* rollOff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_J83B_ROLL_OFF,
                                                     rollOff), 2, "Roll Off");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable J83B Special Settings
 * Purpose:  This function enables use of special J83B coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableJ83BSpecial(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_J83B_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Configure J83B Special Settings
 * Purpose:  This function configures special J83B coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confJ83BSpecialReedSolomon (ViSession instrSession,
                                         ViBoolean reedSolomon)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_J83B_REED_SOLOMON, reedSolomon),
                 2, "Reed Solomon");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure J83B Settings
 * Purpose:  This function configures 8VSB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confJ83BSettings(ViSession   instrSession,
                                         ViInt32 testTSPacket,
                                         ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_TEST_TS_PACKED, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_J83B_PRBS, prbs),
                         3, "PRBS"); 
      
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure J83B Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the J83B signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_ConfigureJ83BPayloadTestStuff (ViSession instrSession,
                                          ViInt32 payloadTestStuff)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_J83B_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 2, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *--------------------------------- MediaFLO --------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure MediaFLO Input Signal
 * Purpose:  This function configures MediaFLO Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confMfloInputSignal (ViSession instrSession,
                                             ViInt32 source,
                                             ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_MFLO_SOURCE,
                            source), 2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT)
    	viCheckParm (rssfc_SetAttributeViInt32(instrSession, "", RSSFC_ATTR_IQC_MFLO_INPUT,
                            input), 3, "Input");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read MediaFLO Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadMediaFLOValues (ViSession instrSession,
                                              ViInt32 value,
                                              ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_MFLO_MEAS_DRATE) || (value > RSSFC_VAL_MFLO_MEAS_LID_OIS))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MFLO_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_MFLO_MEAS_DATA_RATE,
                                                    result), 3, "Result"); 
        break;
        case RSSFC_VAL_MFLO_MEAS_WID_COND:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_WIDE_CONTENT_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MFLO_MEAS_WID_CTRD:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_WIDE_CONTROL_CHANNEL_DATA_RATE,
                                                     result), 3, "Result");
        break;
		case RSSFC_VAL_MFLO_MEAS_WID_OIS:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_WIDE_OIS_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MFLO_MEAS_LID_COND:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_LOCAL_CONTENT_DATA_RATE,
                                                     result), 3, "Result");
        break;
		case RSSFC_VAL_MFLO_MEAS_LID_CTRD:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_LOCAL_CONTROL_CHANNEL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MFLO_MEAS_LID_OIS:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_INPUT_SIGNAL_LOCAL_OIS_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Status MediaFLO Delay Parameters
 * Purpose:  This function reads selected delay parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadStatusMediaFLODelayParameters (ViSession instrSession,
												ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((value < RSSFC_VAL_MFLO_STATUS_SSF_TIME) || (value > RSSFC_VAL_MFLO_STATUS_PPS_TDM))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MFLO_STATUS_SSF_TIME:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_STATUS_SSF_ARRIVAL_TIME,
                                                    result), 3, "Result"); 
        break;
        case RSSFC_VAL_MFLO_STATUS_PPS_TDM:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_STATUS_1PPS_TDM1_ERROR,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Status MediaFLO Count Parameters
 * Purpose:  This function reads selected count parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadStatusMediaFLOCountParameters (ViSession instrSession, 
											ViInt32 value, ViInt32* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((value < RSSFC_VAL_MFLO_STATUS_OUTF_COUNT) || (value > RSSFC_VAL_MFLO_STATUS_SCER_COUNT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MFLO_STATUS_OUTF_COUNT:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_STATUS_OUT_OF_FRAME_COUNT,
                                                    result), 3, "Result"); 
        break;
        case RSSFC_VAL_MFLO_STATUS_SEER_COUNT:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_STATUS_SEQUENCE_ERROR_COUNT,
                                                     result), 3, "Result");
        break;
		case RSSFC_VAL_MFLO_STATUS_SCER_COUNT:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_READ_MEDIAFLO_STATUS_SCHEDULING_ERROR_COUNT,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO Coding
 * Purpose:  This function configures MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureMediaFLOCoding (ViSession instrSession,
                                                 ViInt32 channelBandwidth,
                                                 ViInt32 IFFTMode,
                                                 ViInt32 slotToInterlaceMapping,
                                                 ViInt32 cyclicPrefixLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_MFLO_CHANNEL_BANDWIDTH,
                            channelBandwidth), 2, "Channel Bandwidth");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_IFFT_MODE,
                            IFFTMode), 3, "IFFT Mode");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_SLOT_TO_INTERLACE_MAPPING,
                            slotToInterlaceMapping), 4, "Slot To Interlace Mapping");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_CYCLIC_PREFIX_LENGTH,
                            cyclicPrefixLength), 5, "Cyclic Prefix Length");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO Coding Parameters
 * Purpose:  This function configures MediaFLO coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureMediaFLOCodingParameters (ViSession instrSession,
			  ViInt32 wideAreaIDValue,																	
              ViInt32 localAreaIDValue, ViInt32 wideMACSymbolsFrame,
              ViInt32 localMACSymbolsFrame, ViReal64 networkGeographyDelay)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_WIDE_AREA_ID_VALUE,
                            wideAreaIDValue), 2, "Wide Area ID Value");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_LOCAL_AREA_ID_VALUE,
                            localAreaIDValue), 3, "Local Area ID Value");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_WIDE_MAC_SYMBOLS_FRAME,
                            wideMACSymbolsFrame), 4, "Wide MAC Symbols Frame");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_LOCAL_MAC_SYMBOLS_FRAME,
                            localMACSymbolsFrame), 5, "Local MAC Symbols Frame");

	viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MEDIAFLO_CODING_NETWORK_GEOGRAPHY_DELAY,
                            networkGeographyDelay), 6, "Network Geography Delay");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure MediaFLO PID Settings
 * Purpose:  This function configures MediaFLO PID settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureMediaFLOPIDSettings (ViSession instrSession,
                                                      ViInt32 PID, ViInt32 content,
                                                      ViInt32 controlChannel,
                                                      ViInt32 OIS)
{    
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
   if ((PID < RSSFC_VAL_IQC_MFLO_WIDE_PID) || (PID > RSSFC_VAL_IQC_MFLO_LOCAL_PID))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PID");
        
    switch (PID){
        case RSSFC_VAL_IQC_MFLO_WIDE_PID:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_WIDE_PID_CONTENT,
                            content), 3, "Content ");
			viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_WIDE_PID_CONTROL_CHANNEL,
                            controlChannel), 4, "Control Channel ");
			viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_WIDE_PID_OIS,
                            OIS), 5, "OIS ");
        break;
      
		case RSSFC_VAL_IQC_MFLO_LOCAL_PID:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_LOCAL_PID_CONTENT,
                            content), 3, "Content");
 			viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_LOCAL_PID_CONTROL_CHANNEL,
                            controlChannel), 4, "Control Channel");
  			viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_LOCAL_PID_OIS,
                            OIS), 5, "OIS");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Local OIS System Settings  
 * Purpose:  This function configures MediaFLO local OIS system parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureMediaFLOLocalOISSystemSettings
             (ViSession instrSession, ViBoolean localOISInsertion,
              ViInt32 infrastructureID, ViReal64 localTimeOffset,
              ViBoolean daylightSavingTimeIndicator)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

 	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_LOCAL_OIS_INSERTION,
							localOISInsertion), 2, "Local OIS Insertion");
		
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_INFRASTRUCTURE_ID,
                            infrastructureID), 3, "Infrastructure ID");
	
	viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_LOCAL_TIME_OFFSET,
                            localTimeOffset), 4, "Local Time Offset");
		
	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_DAYLIGHT_SAVINGS_TIME_INDICATOR,
							daylightSavingTimeIndicator), 5, "Daylight Saving Time Indicator");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable MediaFLO Position Pilot Settings  
 * Purpose:  This function enables MediaFLO position pilot parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_EnableMediaFLOPositionPilotSettings
             (ViSession instrSession, ViBoolean positionPilotSymbols)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

 	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_POSITION_PILOT_SYMBOLS,
							positionPilotSymbols), 2, "Position Pilot Symbols");
		
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO Position Pilot Transmitter Settings  
 * Purpose:  This function configures MediaFLO position pilot parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureMediaFLOPositionPilotTransmitterSettings
             (ViSession instrSession, ViInt32 transmitterID, ViInt32 numberOfTransmitters)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TRANSMITTER_ID,
                            transmitterID), 2, "Transmitter ID");
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_NUMBER_OF_TRANSMITTERS,
                            numberOfTransmitters), 3, "Number Of Transmitters");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Enable MediaFLO TX Position Settings
 * Purpose:  This function enables or disables the transmitter position field
 *           information on the PPC channel and sets a number that defines
 *           the transmitter position.
/============================================================================*/
ViStatus _VI_FUNC rssfc_EnableMediaFLOTXPositionSettings (ViSession instrSession,
                                                          ViBoolean TXPositionEnable,
														  ViInt32 TXPosition)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_POSITION_ENABLE, TXPositionEnable),
                         2, "TX Position Enable");
	
	if (TXPositionEnable == VI_TRUE)  
		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_POSITION,
                            TXPosition), 2, "TX Position");
		
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure MediaFLO TX Settings
 * Purpose:  This function configures MediaFLO TX parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureMediaFLOTXSettings (ViSession instrSession,
                                                     ViBoolean TXOffsetEnable,
                                                     ViReal64 TXOffset,
                                                     ViReal64 TXLatitude,
                                                     ViReal64 TXLongitude,
                                                     ViReal64 TXAltitude)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_OFFSET_ENABLE,
							TXOffsetEnable), 3, "TX Offset Enable");
	
	if (TXOffsetEnable == VI_TRUE) {
		viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_OFFSET,
                            TXOffset), 4, "TX Offset");
	}
	
	viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_LATITUDE,
                            TXLatitude), 5, "TX Latitude");
		
	viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_LONGITUDE,
                            TXLongitude), 6, "TX Longitude");
		
	viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_MEDIAFLO_SETTINGS_TX_ALTITUDE,
                            TXAltitude), 7, "TX Altitude");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

	
/*****************************************************************************
 *----------------------------------- T-DMB ---------------------------------*
 *****************************************************************************/


 
/*============================================================================/
 * Function: Configure TDMB Input Signal
 * Purpose:  This function configures T-DMB Input Signal parameters.
/============================================================================*/
 
ViStatus _VI_FUNC rssfc_confTDMBInputSignal (ViSession instrSession,
                                             ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_SOURCE, source),
                         2, "Source");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TDMB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readTDMBValues (ViSession instrSession, ViInt32 value,
                                        ViInt32* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_TDMB_NUM_STREAMS) || (value > RSSFC_VAL_TDMB_MODE_IDENTITY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
   
        case RSSFC_VAL_TDMB_NUM_STREAMS:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_NUMBER_OF_STREAMS,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_TDMB_MODE_IDENTITY:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_MODE_IDENTITY,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TDMB ETI Input
 * Purpose:  This function reads the ETI input signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readTDMBEtiInput (ViSession instrSession,
                                          ViInt32* ETIInput)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_ETI_INPUT,
                                            ETIInput), 2, "ETI Input");   
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read Info STC SSTC
 * Purpose:  This function reads Info STC/SSTC.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readTDMBInfo (ViSession instrSession, ViInt32 subchannel,
                                      ViInt32* SCID, ViInt32* protProfile,
                                      ViInt32* protLevel, ViInt32* dataRate)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";

    if ((subchannel < 1) || (subchannel > 64))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Subchannel"); 
    

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buffer, "S%ld", subchannel);

    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_TDMB_SCID, SCID), 3, "SCID");  
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_TDMB_DATA_RATE, dataRate), 6, "Data Rate");    
	  
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_TDMB_PROT_PROFILE, protProfile), 4, "Prot. Profile");  
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_TDMB_PROT_LEVEL, protLevel), 5, "Prot. Level");  
  
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}



/*============================================================================/
 * Function: Enable DTMB Special Settings
 * Purpose:  This function enables use of special T-DMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableTDMBSpecial (ViSession instrSession,
                                              ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TDMB_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure TDMB Special Settings
 * Purpose:  This function configures special T-DMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTDMBSpecial (ViSession instrSession,
                                         ViInt32 transMode,
                                         ViInt32 selectTransMode,
                                         ViBoolean PRBSTestsignal,
                                         ViInt32 testsignalSCID)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_TRANSMISSION_MODE, transMode),
                         2, "Transmission Mode");
    
    if (transMode == RSSFC_VAL_TDMB_TRAN_MODE_MANUAL)
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_SELECT_TRANSMISSION_MODE, selectTransMode),
                         3, "Select Transmission Mode");
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TDMB_PRBS_TESTSIGNAL, PRBSTestsignal),
                         4, "PRBS Testsignal");                     
    if (PRBSTestsignal) 
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_TESTSIGNAL_SCID, testsignalSCID),
                         5, "Testsignal SCID");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
   
/*============================================================================/
 * Function: T-DMB Configuration
 * Purpose:  This function configures T-DMB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTDMBConfiguration (ViSession instrSession,
                                          ViBoolean TDTMBTI,
                                          ViInt32 TIIMainID,
                                          ViInt32 TIISubID,
                                          ViInt32 TDMBPRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TDMB_TII, TDTMBTI),
                         2, "TDMB TTI");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_TII_MAIN_ID, TIIMainID),
                         3, "TII Main ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_TII_SUB_ID, TIISubID),
                         4, "TII Sub ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TDMB_PRBS, TDMBPRBS),
                         5, "TDMB PRBS");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *----------------------------------- ISDB-T --------------------------------*
 *****************************************************************************/


/*============================================================================/
 * Function: Configure ISDBT Input Source
 * Purpose:  This function configures ISDBT Input Source.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTInputSource (ViSession instrSession,
                                              ViInt32 layer, ViInt32 source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buf1, "%s", layerArr[layer]);

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_SOURCE, source), 
                    3, "Source"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
 
/*============================================================================/
 * Function: Configure ISDBT Input Signal
 * Purpose:  This function configures ISDBT Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTInputSignal (ViSession instrSession,
                                              ViInt32 input, ViBoolean remux,
                                              ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
   
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_INPUT_SIGNAL, input), 
                    2, "Input");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_ISDBT_INPUT_SIGNAL_REMUX, remux),
                         3, "Remux");
	
	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_ISDBT_INPUT_SIGNAL_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*============================================================================/
 * Function: Configure ISDBT Test Signal
 * Purpose:  This function configures ISDBT Test Signal.
/============================================================================*/ 
ViStatus _VI_FUNC rssfc_confISDBTTestSignal (ViSession instrSession,
                                             ViInt32 layer, ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "%s", layerArr[layer]);

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_TEST, testSignal), 
                    3, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
                                            

/*============================================================================/
 * Function: Read ISDBT Input Data Rate
 * Purpose:  This function returns the input data rate.
/============================================================================*/ 
ViStatus _VI_FUNC rssfc_readISDBTInputDataRate (ViSession instrSession,
                                                ViReal64* inputDataRate)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!inputDataRate) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Input Data Rate");
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_INPUT_DATA_RATE,
                                            inputDataRate), 2, "Input Data Rate");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                

/*============================================================================/
 * Function: Read ISDBT Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/ 
ViStatus _VI_FUNC rssfc_readISDBTValues (ViSession instrSession, ViInt32 layer,
                                         ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 
     
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_MEAS_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "%s", layerArr[layer]);

    if (!result) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Result");
    
    switch (value){ 
        case RSSFC_VAL_MEAS_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_MEAS_USEFUL_DATA_RATE,
            result), 4, "Result");                 
        break;

        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_MAX_USEFUL_DATA_RATE,
            result), 4, "Result");                 
        break;                                                    
    }
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure ISDBT System
 * Purpose:  This function configures ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTSystem (ViSession instrSession,
                                         ViInt32 ISDBTSystem)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_CODING_SYSTEM, ISDBTSystem),
                         2, "ISDBT System");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         
 
/*============================================================================/
 * Function: Configure ISDBT Control
 * Purpose:  This function configures the mode of the coder.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureISDBTControl (ViSession instrSession,
                                         ViInt32 control)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_CODING_CONTROL, control),
                         2, "Control");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}  
/*============================================================================/
 * Function: Configure ISDBT Layer Coding
 * Purpose:  This function configures ISDBT coding parameters for layer A, B
 *           or C.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTLayerCoding (ViSession instrSession,
                                              ViInt32 portion, ViInt32 layer,
                                              ViInt32 constellation,
                                              ViInt32 codeRate,
                                              ViInt32 timeInterleaving)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "%s", layerArr[layer]);

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_PORTION, portion), 
                    2, "Portion");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_CONSTELLATION, constellation), 
                    4, "Constellation");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_CODE_RATE, codeRate), 
                    5, "Code Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_TIME_INTERLEAVING, timeInterleaving), 
                    6, "Time Interleaving");                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure ISDBT Coding
 * Purpose:  This function configures ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTCoding (ViSession instrSession,
                                         ViInt32 ISDBTMode, ViInt32 guardInterval)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_MODE, ISDBTMode),
                         2, "ISDBT Mode"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_GUARD_INTERVAL, guardInterval),
                         3, "Guard Interval");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         
                                         
 
/*============================================================================/
 * Function: Configure ISDBT Sub Channel
 * Purpose:  This function sets the sub-channel of the ISDB-TSB system.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTSubChannel (ViSession instrSession,
                                         ViInt32 subChannel)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_CODING_SUB_CHANNEL, subChannel),
                         2, "Sub-Channel"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         
 
/*============================================================================/
 * Function: Configure ISDBT Segments
 * Purpose:  This function sets the segments for layer A, B or C.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTSegments (ViSession instrSession,
                                           ViInt32 layer, ViInt32 segments)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "%s", layerArr[layer]);

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_SEGMENTS, segments), 
                    3, "Segments");
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read ISDBT Bandwidth
 * Purpose:  This function queries the bandwidth of the ISDB-T signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readISDBTBandwidth (ViSession instrSession,
                                            ViReal64* bandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!bandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Bandwidth");
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_BANDWIDTH,
                                            bandwidth), 2, "Bandwidth");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                             


/*============================================================================/
 * Function: Enable ISDBT Special Settings
 * Purpose:  This function enables use of special ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableISDBTSpecial (ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_IQC_ISDBT_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                               

                                                         
/*============================================================================/
 * Function: Configure ISDBT Special Segment Settings
 * Purpose:  This function configures AC data for special ISDBT modulation
 *           coding.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTSpecialAcDataSettings (ViSession instrSession,
                                                        ViInt32 ACData_AC1,
                                                        ViInt32 ACData_AC2)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_AC_DATA1, ACData_AC1), 
                 2, "AC Data AC1");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_AC_DATA2, ACData_AC2), 
                 3, "AC Data AC2");                     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                           
  
/*============================================================================/
 * Function: Enable ISDBT Special Alert Broadcast
 * Purpose:  This function switches the alert broadcasting flag on or off.
/============================================================================*/
ViStatus _VI_FUNC rssfc_EnableISDBTSpecialAlertBroadcast (ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ISDBT_SPECIAL_SETTINGS_ALERT_BROADCAST, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}  

/*============================================================================/
 * Function: Configure ISDBT Special Settings
 * Purpose:  This function configures special ISDBT coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureISDBTSpecialSettings (ViSession instrSession,
                                                       ViBoolean residualCarrierShift,
                                                       ViInt32 TXParamSwitchingIndicator,
                                                       ViInt32 TMCCNext)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ISDBT_SPECIAL_SETTINGS_RESIDUAL_CARRIER_SHIFT, residualCarrierShift),
                        2, "Residual Carrier Shift");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ISDBT_SPECIAL_SETT_TX_PARAM_SWITCHING_IND, TXParamSwitchingIndicator), 
                        3, "TX Param witching Indicator");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ISDBT_SPECIAL_SETTINGS_TMCC_NEXT, TMCCNext), 
                        4, "TMCC Next");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}  

/*============================================================================/
 * Function: Configure ISDBT Settings
 * Purpose:  This function configures ISDBT coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confISDBTSettings (ViSession instrSession,
                                           ViInt32 layer, ViInt32 testTSPacket,
                                           ViInt32 prbs)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "%s", layerArr[layer]);

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_IQC_ISDBT_TEST_TS_PACKED, testTSPacket), 
                    3, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_ISDBT_PRBS, prbs), 
                    4, "PRBS");                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           


/*============================================================================/
 * Function: Configure ISDBT Payload Test Stuff
 * Purpose:  This function allows selection of the payload in the TS packets
 *           of the ISDBT signal.
/============================================================================*/    
ViStatus _VI_FUNC rssfc_ConfigureISDBTPayloadTestStuff (ViSession instrSession,
                                          ViInt32 layer, ViInt32 payloadTestStuff)
{    
    ViStatus    error = VI_SUCCESS;  
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((layer < RSSFC_VAL_IQC_ISDBT_LAYER_A) || (layer > RSSFC_VAL_IQC_ISDBT_LAYER_C))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Layer"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "%s", layerArr[layer]);

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_ISDBT_SETTINGS_PAYLOAD_TEST_STUFF,
                    payloadTestStuff), 3, "Payload Test/Stuff");                  
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *--------------------------------- DVB-T2 ----------------------------------*
 *****************************************************************************/
/*===========================================================================*/
/* Function: Configure DVBT2 Input Signal
/* Purpose:  This function configures DVB-T2 Input Signal parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2InputSignal (ViSession instrSession,
                                              ViBoolean t2MIInterface,
                                              ViInt32 t2MISource,
                                              ViInt32 numberOfPLP,
                                              ViInt32 t2MIInput)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
        
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_INTERFACE_STATE,
						t2MIInterface), 2, "T2-MI Interface");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_SOURCE, t2MISource),
                         3, "T2-MI Source");
   
	if (t2MISource == RSSFC_VAL_DVBT2_MI_SOUR_EXT)
			viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_INTERFACE_INPUT,
						t2MIInput), 5, "T2-MI Input");
	
	if (t2MIInterface == VI_FALSE) 
			viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_NUMBER_OF_PLP, numberOfPLP),
                         4, "Number Of PLP"); 
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Input T2 MI PID
/* Purpose:  This function configures PID used for T2-MI decapsulation from
/*  		 the MPEG transport stream.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2InputT2MIPID(
	ViSession	instrSession, ViInt32 t2MIPID)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_PID, t2MIPID),
                         2, "T2-MI PID");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Input T2 MI SID
/* Purpose:  This function configures T2-MI stream ID used when transmitting
/*           a composite signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2InputT2MISID(
	ViSession	instrSession, ViInt32 t2MISID)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_SID, t2MISID),
                         2, "T2-MI SID");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Reset DVBT2 T2 MI Analyzer Log File
/* Purpose:  This function resets the T2-MI analyzer log file.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ResetDVBT2T2MIAnalyzerLogFile(ViSession	instrSession)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckErr (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_RESET_T2_MI_ANALYZER_LOG_FILE, ""));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 T2 MI Analyzer
 * Purpose:  This function reads the current error message from the T2-MI
/*           analyzer.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBT2T2MIAnalyzer (ViSession instrSession,
                                                       ViChar  t2MIAnalyzer[])
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "\0";

    checkErr( Rs_LockSession (instrSession, VI_NULL)); 
    

    if (t2MIAnalyzer == 0) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "T2 MI Analyzer");
    viCheckErr (rssfc_GetAttributeViString (instrSession, "", RSSFC_ATTR_DVBT2_INPUT_SIGNAL_T2_MI_ANALYZER,
                                            RSSFC_IO_BUFFER_SIZE, buffer));
    strcpy (t2MIAnalyzer, buffer);                                           
        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

		   
/*===========================================================================*/
/* Function: Configure DVBT2 PLP
/* Purpose:  This function configures DVB-T2 PLP parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configureDVBT2Plp(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	inputFormat,
	ViInt32	source, 
	ViInt32	input,
	ViBoolean	stuffing
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";   

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);     
    
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_INPUT_FORMAT, inputFormat),
                         3, "Input Format");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_SOURCE, source ),
                         4, "Source"); 
    if (source == RSSFC_IQC_SIG_SOUR_EXT)
		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_INPUT, input),
                         5, "Input"); 
    if (source != RSSFC_IQC_SIG_SOUR_TEST)	
    	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_STUFFING, stuffing),
                         6, "Stuffing");                          
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Read DVBT2 Values
/* Purpose:  This function reads selected coding value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readDVBT2Values(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	value,
	ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);
   
     switch (value){
        case RSSFC_VAL_MAX_USEFUL_DRATE:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_MAX_USEF_DR, 
	            result), 4, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_MEAS_DR, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_USEFUL_DRATE:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_USEFUL_DR, 
                result), 4, "Result");
        break;
		
		
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
	}

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Test Signal
/* Purpose:  This function configures DVB-T2 Test Signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2TestSignal(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	testSignal
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);
	
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         3, "Test Signal");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}
/*===========================================================================*/
/* Function: Read DVBT2 Mode Stream Adaptation Values
/* Purpose:  This function reads selected coding value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBT2ModeStreamAdaptationValues
             (ViSession instrSession, ViInt32 plp, ViInt32* PLPGroupID,
              ViInt32* otherInBandPLP)

{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";  

 	if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);
	
	if (!PLPGroupID) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "PLP Group ID");
	
	if (!otherInBandPLP) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Other In Band PLP");  
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_GROUP_ID, 
               PLPGroupID), 3, "PLP Group ID");
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession,buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_OTHER_IN_BAND_PLP, 
               otherInBandPLP), 4, "Other In Band PLP");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Mode Stream Adaptation
/* Purpose:  This function configures DVB-T2 Mode Stream Adaptation 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2ModeStreamAdaptation(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	plpID,
	ViInt32	plpType,
	ViInt32	bbMode,
	ViInt32 issy,
	ViBoolean nullPacketDel,
	ViBoolean inBandSignal,
	ViInt32 cmType
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_ID, plpID),
                         3, "PLP ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_PLP_TYPE, plpType),
                         4, "PLP Type");                          
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_BB_MODE, bbMode),
                         5, "BB Mode");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_ISSY, issy),
                         6, "ISSY");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_NULL_PACKET_DEL, nullPacketDel),
                         7, "Null Packet Del");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL, inBandSignal),
                         8, "In Band Signal");
 	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_CM_TYPE, cmType),
                         9, "CM Type");	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}									

/*===========================================================================*/
/* Function: Configure DVBT2 Mode Stream Adaptation In Band Signal
/* Purpose: This function configures DVB-T2 mode stream In Band Signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2ModeStreamAdaptationInBandSignal
             (ViSession instrSession, ViInt32 plp, ViBoolean inBandSignalA,
              ViBoolean inBandSignalB)

{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL_A, inBandSignalA),
                         3, "In Band Signal A");
	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBT2_MODE_STREAM_ADAPT_IN_BAND_SIGNAL_B, inBandSignalB),
                         4, "In Band Signal B");
                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 BICM Parameters
/* Purpose:  This function configures DVB-T2 Bit Interleaved Coding and 
/*           Modulation subitem parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2BICMParameters(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	fecFrame,
	ViInt32	codeRate,
	ViInt32	constellation,
	ViBoolean constellRotation
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_FEC_FRAME, fecFrame),
                         3, "FEC Frame");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_CODE_RATE, codeRate),
                         4, "Code Rate");                          
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_CONSTELL, constellation),
                         5, "Constellation");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_CONSTELL_ROTATION, constellRotation),
                         6, "Constell Rotation");
                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 BICM Time Interleaver Parameters
/* Purpose:  This function configures DVB-T2 Bit Interleaved Coding and 
/*           Modulation subitem parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2BICMTimeInterleaverParameters(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	timeInterleaverType,
	ViInt32	frameIntIJump,
	ViInt32	timeInterleaverLength
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbt2PlpArr[plp]);
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_TIME_INTERL_TYPE, timeInterleaverType),
                         3, "Time Interleaver Type");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_FRAME_INT_I_JUMP, frameIntIJump),
                         4, "Frame Int I Jump");                          
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_TIME_INTERL_LENGTH, timeInterleaverLength),
                         5, "Time Interleaver Length");  
                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBT2 BICM FEC Blocks Per IF
/* Purpose:  This function queries the FEC blocks per interleaving frame.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readDVBT2BICMFECBlocksPerIF (ViSession instrSession,
                                                     ViInt32 PLP, ViInt32 block,
                                                     ViInt32* value
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((PLP < 1) || (PLP > 16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP");
 	
	if ((block < RSSFC_VAL_IQC_DVBT2_FEC_BLOCK) || (block > RSSFC_VAL_IQC_DVBT2_FEC_MAX_BLOCK))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block");
	
	sprintf (buf1, "PLP%s", dvbt2PlpArr[PLP]); 
	
    switch (block){
        case RSSFC_VAL_IQC_DVBT2_FEC_BLOCK:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_FEC_BLOCKS_PER_IF,
                                                     value), 4, "Value");
        break;
        case RSSFC_VAL_IQC_DVBT2_FEC_MAX_BLOCK:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBT2_BICM_FEC_MAX_BLOCKS_PER_IF,
                                                     value), 4, "Value");
        break;
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
	}  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}		   


/*===========================================================================*/
/* Function: Configure DVBT2 Framing OFDM Parameters
/* Purpose:  This function configures FRAMING + OFDM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2BICMFramingOFDMParameters(
	ViSession	instrSession,
	ViInt32	channelBandwidth,
	ViInt32 bandwidthVariation,
	ViInt32	fftSize,
	ViInt32	guardInterval,
	ViInt32	pilotPattern
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_BANDWIDTH_VARIATION, bandwidthVariation),
                         3, "Bandwidth Variation");	
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_FFT_SIZE, fftSize),
                         4, "FFT Size");                          
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_GUARD_INTERVAL, guardInterval),
                         5, "Guard Interval");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_PILOT_PATTERN, pilotPattern),
                         6, "Pilot Pattern");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBT2 Framing OFDM Symbols Per T2 Frame
/* Purpose:  This function queries the OFDM symbols per T2 frame (L_F).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readDVBT2FramingOFDMSymbolsPerT2Frame
             (ViSession instrSession, ViInt32* OFDMSymbolsPerT2Frame)
{
	ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (!OFDMSymbolsPerT2Frame) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "OFDM Symbols Per T2 Frame");
															   
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_OFDM_SYMBOLS_PER_T2_FRAME, OFDMSymbolsPerT2Frame),
                         2, "OFDM Symbols Per T2 Frame"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBT2 Framing OFDM Used Bandwidth
/* Purpose:  This function queries the used channel bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readDVBT2FramingOFDMUsedBandwidth(
	ViSession	instrSession,
	ViReal64*	usedBandwidth
)
{
	ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (!usedBandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Used Bandwidth");
															   
	viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_USED_BANDWIDTH, 
               usedBandwidth), 2, "Used Bandwidth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Framing OFDM
/* Purpose:  This function configures DVB-T2 Frames parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2FramingOFDM(
	ViSession	instrSession,
	ViInt32	t2FramesPerSuperFrame,
	ViInt32 dataSymbolsPerT2Frame,
	ViInt32	subslicesPerT2Frame
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_T2_FRAMES_PER_SUPER_FRAME, t2FramesPerSuperFrame),
                         2, "T2 Frames Per Super Frame");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_DATA_SYMBOLS_PER_T2_FRAME, dataSymbolsPerT2Frame),
                         3, "Data Symbols Per T2 Frame");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_FRAMING_SUBSLICES_PER_T2_FRAME, subslicesPerT2Frame),
                         4, "Subslices Per T2 Frame");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System Network
/* Purpose:  This function configures T2 system network mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2TransmissionSystemNetwork
             (ViSession instrSession, ViInt32 networkMode)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_NETWORK_MODE, networkMode),
                         2, "Network Mode");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System
/* Purpose:  This function configures transmission system parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2TransmissionSystem (ViSession instrSession,
                                                     ViInt32 networkMode,
													 ViInt32 transmissionSystem,
                                                     ViInt32 peakToAvragePowerRatio,
                                                     ViBoolean futureExtensionFrames,
                                                     ViBoolean timeFrequencySlicing
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_NETWORK_MODE, networkMode),
                         2, "Network Mode"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_TRANSMISSION_SYSTEM, transmissionSystem),
                         3, "Transmission System");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_PEAK_TO_AVRAGE_POWER_RATIO, peakToAvragePowerRatio),
                         4, "Peak To Avrage Power Ratio");                          
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES, futureExtensionFrames),
                         5, "Future Extension Frames");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_TIME_FREQUENCY_SLICING, timeFrequencySlicing),
                         6, "Time Frequency Slicing");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 MISO Group Mode
/* Purpose:  This function configures the MISO group operation mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2MISOGroupMode (ViSession instrSession,
                                                     ViInt32 groupMode)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_MISO_GROUP_MODE, groupMode),
                         2, "Group Mode");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System Network
/* Purpose:  This function configures T2 system profile mode.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2TransmissionProfileMode
             (ViSession instrSession, ViInt32 profileMode)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_PROFILE_MODE, profileMode),
                         2, "Profile Mode");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission Future Extension Frames
/* Purpose:  This function configures Future Extension Frames parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2TransmissionFutureExtensionFrames
             (ViSession instrSession, ViInt32 FEFLength, ViInt32 FEFType,
              ViInt32 FEFInterval)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_LENGTH, FEFLength),
                         2, "FEF Length");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_TYPE, FEFType),
                         3, "FEF Type");                          
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_INTERVAL, FEFInterval),
                         4, "FEF Interval");                          
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission Future Extension Payload
/* Purpose:  This function configures Future Extension Frames payload.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2TransmissionFutureExtensionPayload
             (ViSession instrSession, ViInt32 FEFPayload)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_FUTURE_EXTENSION_FRAMES_PAYLOAD, FEFPayload),
                         2, "FEF Payload");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure DVBT2 Transmission L Parameters
/* Purpose:  This function configures transmission system L parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2TransmissionLParam(
	ViSession	instrSession,
	ViInt32 l1T2Version,
	ViInt32	lPostModulation,
	ViBoolean	lRepetition,
	ViBoolean	lPostExtension,
	ViInt32	numAuxStreams,
	ViBoolean	lRFSignalling
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	 
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L1_T2_VERSION, l1T2Version),
                         2, "L1 T2 Version");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L_POST_MODULATION, lPostModulation),
                         3, "L Post Modulation");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L_REPETITION, lRepetition),
                         4, "L Repetition");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L_POST_EXTENSION, lPostExtension),
                         5, "L Post Extension");  
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_NUM_AUX_STREAMS, numAuxStreams),
                         6, "Num Aux Streams");                          
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L_RF_SIGNALLING, lRFSignalling),
                         7, "L RF Signalling");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBT2 Transmission System Identification
/* Purpose:  This function configures transmission system identification 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2TransmissionSystemIdentification(
	ViSession	instrSession,
	ViInt32	cellID,
	ViInt32	networkID,
	ViInt32	systemID,
	ViUInt16 txIDAvailability
)
{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_CELL_ID, cellID),
                         2, "Cell ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_NETWORK_ID, networkID),
                         3, "Network ID");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_ID, systemID),
                         4, "System ID");  
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_TX_ID_AVAILABILITY, txIDAvailability),
                         5, "Tx ID Availability");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Transmission L Parameters L1T2
/* Purpose:  This function configures L1 post scrambling according to T2 and
/*           T2 base lite signaling version 1.3.1 ("L1 T2 VERSION").
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2TransmissionLParametersL1T2(
	ViSession	instrSession, ViBoolean l1PostScrambled, ViBoolean t2BaseLite)

{
 ViStatus    error = VI_SUCCESS;
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L1_POST_SCRAMBLED, l1PostScrambled),
                         2, "L1 Post Scrambled");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_T2_BASE_LITE, t2BaseLite),
                         3, "T2 Base Lite");  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 T2 System L1 Signalling Frequency
 * Purpose:  This function queries the frequency signaled via L1 signaling.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBT2T2SystemL1SignallingFrequency
             (ViSession instrSession, ViReal64* frequency)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    if (!frequency) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Frequency");
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_T2_SYSTEM_L1_SIGNALLING_FREQUENCY,
                                            frequency), 2, "Frequency");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 SFN Delay Parametrs
/* Purpose:  This function configures SFN delay parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2SFNDelayParameters (ViSession instrSession,
                                                          ViInt32 delayParameter,
                                                          ViReal64 timeValue)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((delayParameter < RSSFC_VAL_DVBT2_SFN_MAX_DEVIATION_TIME) || (delayParameter > RSSFC_VAL_DVBT2_SFN_STATIC_DELAY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
     switch (delayParameter) {
		 case RSSFC_VAL_DVBT2_SFN_MAX_DEVIATION_TIME:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_MAX_DEVIATION_TIME,
                                                     timeValue), 3, "Time Value");
        break;
		 case RSSFC_VAL_DVBT2_SFN_STATIC_DELAY:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_STATIC_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
  
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Configure DVBT2 SFN Mute P1 Symbol
/* Purpose:  This function mutes the P1 symbol of the first T2 frame in a
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2SFNMuteP1Symbol (ViSession instrSession,
                                                       ViBoolean muteP1SymbolOfSuperFrame)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBT2_SFN_MUTE_FIRST_P1_OF_SUPER_FRAME,
		muteP1SymbolOfSuperFrame), 2, "Mute P1 Symbol Of Super Frame");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 SFN Timestamp Offset
/* Purpose:  This function sets the UTC/GPS leap seconds offset.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2SFNTimestampOffset (ViSession instrSession,
                                                       ViInt32 timestampOffset)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_TIMESTAMP_MODE_OFFSET,
		timestampOffset), 2, "Timestamp Offset");
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBT2 SFN Delay Parameters
/* Purpose:  This function queries selected delay value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBT2SFNDelayParameters (ViSession instrSession,
                                                     ViInt32 delayParameter,
													 ViReal64 *timeValue)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((delayParameter < RSSFC_VAL_DVBT2_SFN_MAX_DEVIATION_TIME) || (delayParameter > RSSFC_VAL_DVBT2_SFN_PROCESS_DELAY))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (delayParameter) {
		 case RSSFC_VAL_DVBT2_SFN_MAX_DEVIATION_TIME:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_MAX_DEVIATION_TIME,
                                                     timeValue), 3, "Time Value");
        break;
		 case RSSFC_VAL_DVBT2_SFN_STATIC_DELAY:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_STATIC_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
        case RSSFC_VAL_DVBT2_SFN_PROCESS_DELAY:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_PROCESS_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
        case RSSFC_VAL_DVBT2_SFN_DYNAMIC_DELAY:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_DYNAMIC_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
		case RSSFC_VAL_DVBT2_SFN_TOTAL_DELAY:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_TOTAL_DELAY,
                                                     timeValue), 3, "Time Value");
        break;
     }           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure DVBT2 SFN Update Timestamp
 * Purpose:  This function updates the UTC time and date reference.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBT2SFNUpdateTimestamp (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_DVBT2_SFN_UPDATE_TIMESTAMP, "")); 

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 SFN Timestamp Mode
 * Purpose:  This function queries the type of the currently received T2-MI
 *           timestamps.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBT2SFNTimestampMode
             (ViSession instrSession, ViInt32* timestampMode)

{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    if (!timestampMode) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Frequency");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_TIMESTAMP_MODE,
                                            timestampMode), 2, "Timestamp Mode");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read DVBT2 SFN Timestamp Parameters
 * Purpose:  This function queries the currently received T2-MI timestamps
 *           parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBT2SFNTimestampParameters (ViSession instrSession,
                                                         ViInt32* UTCDate,
                                                         ViInt32* UTCTime,
                                                         ViInt32* secondsSince2000)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    if (!UTCDate) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "UTC Date");
	if (!UTCTime) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "UTC Time"); 
	if (!secondsSince2000) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Seconds Since 2000"); 
			
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_TIMESTAMP_MODE_UTC_DATE,
                                            UTCDate), 2, "UTC Date");
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_TIMESTAMP_MODE_UTC_TIME,
                                            UTCTime), 3, "UTC Time");
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SFN_TIMESTAMP_MODE_SECONDS_SINCE_2000,
                                            secondsSince2000), 4, "Seconds Since 2000");
  
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBT2 Settings
/* Purpose:  This function configures DVB-T2 coding settings.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confDVBT2Settings(
	ViSession	instrSession,
	ViInt32	testTSPacket,
	ViInt32	payloadTestTSPacket,
	ViInt32	prbs
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SETTINGS_TEST_TS_PACKET, testTSPacket),
                         2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SETTINGS_PAYLOAD_TEST_TS_PACKET, payloadTestTSPacket),
                         3, "Payload Test TS Packet"); 
  
    switch (payloadTestTSPacket){
        case  RSSFC_VAL_IQC_PRBS:
      		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBT2_SETTINGS_PRBS, prbs),
                         4, "PRBS");    
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*****************************************************************************
 *---------------------------- Analog TV ------------------------------------*
 *****************************************************************************/



/*============================================================================/
 * Function: Configure Analog TV Audio Input Signal
 * Purpose:  This function selects the audio source of the R&S SFE.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvAudioInputSignal (ViSession instrSession,
                                                      ViInt32 audioInputSignal)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_AUDIO_INPUT_SIGNAL,
                            audioInputSignal), 2, "Audio Input Signal");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Analog TV Audio
 * Purpose:  This function configures analog TV audio parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_configAnalogTvAudio (ViSession instrSession,
                                             ViInt32 frequencyParameter,
                                             ViReal64 frequencyValue,
                                             ViInt32 levelParameter,
                                             ViReal64 levelValue)
{                                             
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((frequencyParameter < RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_MONO) || (frequencyParameter > RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_CH2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Frequency Parameter");
        
    if ((levelParameter < RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_MONO) || (levelParameter > RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_CH2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Level Parameter");        
  
    switch (frequencyParameter){
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_MONO:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_FREQ_SOUND_MONO,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_LEFT:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_FREQ_SOUND_LEFT,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_RIGHT:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_FREQ_SOUND_RIGHT,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_CH1:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_FREQ_SOUND_CH1,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_CH2:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_FREQ_SOUND_CH2,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
     }           

    switch (levelParameter){
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_MONO:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_LEVEL_SOUND_MONO,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_LEFT:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_LEVEL_SOUND_LEFT,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_RIGHT:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_LEVEL_SOUND_RIGHT,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_CH1:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_LEVEL_SOUND_CH1,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_CH2:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_LEVEL_SOUND_CH2,
                                                    levelValue), 5, "Level Value"); 
        break;
     }           
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Configure Analog TV Audio AF
 * Purpose: This function switches audio channel 1 or 2 ON or OFF.
/============================================================================*/
ViStatus _VI_FUNC rssfc_configAnalogTvAudioAf (ViSession instrSession,
                                               ViInt32 audioChannel,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((audioChannel < RSSFC_VAL_ATV_AUDIO_GEN_AF1) || (audioChannel > RSSFC_VAL_ATV_AUDIO_GEN_AF2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Audio Channel");
        
    switch (audioChannel){
        case RSSFC_VAL_ATV_AUDIO_GEN_AF1:
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_AUDIO_AF_1, state),
                         3, "State"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_GEN_AF2:
             viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_AUDIO_AF_2, state),
                         3, "State"); 
        break;
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                  


/*============================================================================/
 * Function: Load Analog TV Audio Library
 * Purpose:  This function selects the audio player file.
/============================================================================*/
ViStatus _VI_FUNC rssfc_loadAnalogTvAudioLibrary(ViSession instrSession,
                                                    ViString loadLibrary)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViString (instrSession, "",
        RSSFC_ATTR_ANALOG_TV_AUDIO_LOAD_AUDIO_PLAYER_FILE, 
                loadLibrary), 2, "Load Library");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Analog TV Audio Library Catalog
 * Purpose:  This function reads out the files in the default directory.
/============================================================================*/
ViStatus _VI_FUNC rssfc_analogTvAudioLibraryCatalog (ViSession instrSession,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar output[],
                                            ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory));
    
    viCheckErr( viPrintf(instrSession, ":IQC:RATV:APL:LIBR:CAT?\n"));
    
    viCheckErr( rssfc_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfc_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) 
    {
        len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
        strncpy(output, pbuffer, len);
        output[len] = '\0';
    }
    
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfc_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
                                            

/*============================================================================/
 * Function: Analog TV Audio Library Catalog Length
 * Purpose:  This function queries the number of files in the default
 *           directory for audio player files (D:\AUDIOPLAYER).
/============================================================================*/
ViStatus _VI_FUNC rssfc_analogTvAudioLibraryCatalogLength (ViSession instrSession,
                                                  ViString directory,
                                                  ViInt32* catalogLength)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Catalog Length");
    
    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
        
    sprintf (buffer,":RATV:APL:LIBR:CAT:LENG?");
    
    viCheckErr( viQueryf (instrSession, buffer, "%ld", catalogLength));
    }   
    
    viCheckErr(rssfc_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    


/*============================================================================/
 * Function: Configure Analog TV NICAM Source
 * Purpose:  This function selects the NICAM signal source.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvNICAMSource (ViSession instrSession,
                                                 ViInt32 source,
                                                 ViInt32 NICAMSoundMode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_SOURCE,
                            source), 2, "Source");
    if (source == RSSFC_VAL_IQC_ATV_NICAM_SOUR_NGENER)
         viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_SOUND_MODE,
                            NICAMSoundMode), 3, "NICAM Sound Mode");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
         
/*============================================================================/
 * Function: Configure Analog TV Audio AF
 * Purpose: This function switches NICAM audio channel 1 or 2 ON or OFF.
/============================================================================*/
ViStatus _VI_FUNC rssfc_configAnalogTvNICAMAf (ViSession instrSession,
                                               ViInt32 audioChannel,
                                               ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((audioChannel < RSSFC_VAL_ATV_AUDIO_GEN_AF1) || (audioChannel > RSSFC_VAL_ATV_AUDIO_GEN_AF2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Audio Channel");
        
    switch (audioChannel){
        case RSSFC_VAL_ATV_AUDIO_GEN_AF1:
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_AF_1, state),
                         3, "State"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_GEN_AF2:
             viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_AF_2, state),
                         3, "State"); 
        break;
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                  


/*============================================================================/
 * Function: Configure Analog TV Audio
 * Purpose:  This function configures analog TV NICAM parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_configAnalogTvNICAM (ViSession instrSession,
                                             ViInt32 frequencyParameter,
                                             ViReal64 frequencyValue,
                                             ViInt32 levelParameter,
                                             ViReal64 levelValue)
{                                             
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((frequencyParameter < RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_MONO) || (frequencyParameter > RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_CH2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Frequency Parameter");
        
    if ((levelParameter < RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_MONO) || (levelParameter > RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_CH2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Level Parameter");        
  
    switch (frequencyParameter){
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_MONO:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_FREQUENCY_SOUND_MONO,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_LEFT:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_FREQUENCY_SOUND_LEFT,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_RIGHT:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_FREQUENCY_SOUND_RIGHT,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_CH1:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_FREQUENCY_SOUND_CHANNEL_1,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_FREQ_CH2:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_FREQUENCY_SOUND_CHANNEL_2,
                                                    frequencyValue), 3, "Frequency Value"); 
        break;
     }           

    switch (levelParameter){
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_MONO:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_LEVEL_SOUND_MONO,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_LEFT:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_LEVEL_SOUND_LEFT,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_RIGHT:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_LEVEL_SOUND_RIGHT,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_CH1:
              viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_LEVEL_SOUND_CHANNEL_1,
                                                    levelValue), 5, "Level Value"); 
        break;
        case RSSFC_VAL_ATV_AUDIO_SOUND_LEVEL_CH2:
               viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_NICAM_LEVEL_SOUND_CHANNEL_2,
                                                    levelValue), 5, "Level Value"); 
        break;
     }           
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Analog TV Video Input Signal
 * Purpose:  This function selects the audio source of the R&S SFE.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvVideoInputSignal (ViSession instrSession,
                                                      ViInt32 videoInputSignal)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_VIDEO_INPUT_SIGNAL,
                            videoInputSignal), 2, "Video Input Signal");
                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 


/*============================================================================/
 * Function: Configure Analog TV Video
 * Purpose:  This function configures the test pattern in the VIDEO GENERATOR
 *           menu.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvVideo (ViSession instrSession,
                                           ViInt32 visionPicture)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_VISION_PICTURE, visionPicture),
                         2, "Vision Picture");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*============================================================================/
 * Function: Load Analog TV Video Library
 * Purpose:  This function selects video pattern from ATVVIDEOLIB.
/============================================================================*/
ViStatus _VI_FUNC rssfc_loadAnalogTvVideoLibrary(ViSession instrSession,
                                                    ViString loadLibrary)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_ATV_LOAD_VIDEO_LIB, 
                loadLibrary), 2, "Load Library");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Analog TV Video Library Catalog
 * Purpose:  This function reads out the files in the default directory.
/============================================================================*/
ViStatus _VI_FUNC rssfc_analogTvVideoLibraryCatalog (ViSession instrSession,
                                            ViString directory, ViInt32 bufferSize,
                                            ViChar output[],
                                            ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0, len = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory));
    
    viCheckErr( viPrintf(instrSession, ":IQC:RATV:VIDG:LIBR:CAT?\n"));
    
    viCheckErr( rssfc_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfc_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) 
    {
        len = (ViUInt32) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize);
        strncpy(output, pbuffer, len);
        output[len] = '\0';
    }
    
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfc_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}
                                            

/*============================================================================/
 * Function: Analog TV Video Library Catalog Length
 * Purpose:  This function reads out the number of files in the default
 *           directory (D:\ATVVIDEOLIB).
/============================================================================*/
ViStatus _VI_FUNC rssfc_analogTvVideoLibraryCatalogLength (ViSession instrSession,
                                                  ViString directory,
                                                  ViInt32* catalogLength)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Catalog Length");
    
    viCheckErr( viPrintf (instrSession, ":MMEM:CDIR '%s'", directory)); 
        
    sprintf (buffer,":IQC:RATV:VIDG:LIBR:CAT:LENG?");
    
    viCheckErr( viQueryf (instrSession, buffer, "%ld", catalogLength));
    }   
    
    viCheckErr(rssfc_CheckStatus (instrSession));
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                    


/*============================================================================/
 * Function: Configure Analog TV Video Vision Picture
 * Purpose:  This function configures the sound modulation mode.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvVideoVisionPicture (ViSession instrSession,
                                                        ViInt32 mode)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_SOUND_MODUL_MODE, mode),
                         2, "Mode");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                        
    

/*============================================================================/
 * Function: Read Analog TV Values
 * Purpose:  This function reads selected value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readAnalogTvValues (ViSession instrSession,
                                            ViInt32 value, ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((value < RSSFC_VAL_ATV_SOUND_FREQ_DEV_MONO) || (value > RSSFC_VAL_ATV_SOUND_FREQ_DEV_PILOT))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_ATV_SOUND_FREQ_DEV_MONO:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SOUND_MODUL_FREQ_DEV_MONO,
                                                    result), 3, "Result"); 
        break;
        case RSSFC_VAL_ATV_SOUND_FREQ_DEV_SUBCAR1:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SOUND_MODUL_FREQ_DEV_SUBCAR1,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_ATV_SOUND_FREQ_DEV_SUBCAR2:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SOUND_MODUL_FREQ_DEV_SUBCAR2,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_ATV_SOUND_FREQ_DEV_PILOT:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SOUND_MODUL_FREQ_DEV_PILOT,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 
/*============================================================================/
 * Function: Enable Analog TV Special Settings
 * Purpose:  This function enables use of special Analog TV coding
 *           parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actEnableAnalogTvSpecial (ViSession instrSession,
                                                  ViBoolean state)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                                  

/*============================================================================/
 * Function: Configure Analog TV Preemphasis
 * Purpose:  This function configures the preemphasis for the AF sound
 *           channels.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvPreemphasis (ViSession instrSession,
                                                 ViInt32 preemphasis)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_PREEMPHASIS, preemphasis),
                         2, "Preemphasis");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
                                                  

/*============================================================================/
 * Function: Configure Analog TV Pilot
 * Purpose:  This function configures the pilot carrier on sound subcarrier 2.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvPilot (ViSession instrSession,
                                           ViBoolean pilot,
                                           ViReal64 nomFreqDevPilot)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_PILOT, pilot),
                         2, "Pilot");
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_PILOT, nomFreqDevPilot),
                         3, "Nom. Freq. Dev. Pilot");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                           
                                           
/*============================================================================/
 * Function: Configure Analog TV Subcarrier Settings
 * Purpose:  This function configures special Analog TV coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvSubcarrierSettings (ViSession instrSession,
                                                        ViInt32 subcarrierParameter,
                                                        ViReal64 nomFreqDev,
                                                        ViBoolean subcarrier,
                                                        ViReal64 frequencySubcarrier,
                                                        ViReal64 levelSubcarrier)
{                                             
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((subcarrierParameter < RSSFC_VAL_ATV_SOUND_SPEC_SUBCAR_MONO) || (subcarrier > RSSFC_VAL_ATV_SOUND_SPEC_SUBCAR2))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Subcarrier");
 

    switch (subcarrierParameter){
        case RSSFC_VAL_ATV_SOUND_SPEC_SUBCAR_MONO:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_MONO,
                                                    nomFreqDev), 3, "Nom. Freq. Dev."); 
            
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_SUBCARRIER,
                                                    subcarrier), 4, "Subcarrier State");
                                                    
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_FREQ_SUBCARRIER,
                                                    frequencySubcarrier), 5, "Frequency Subcarrier");
        
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_LEVEL_SUBCARRIER,
                                                    levelSubcarrier), 6, "Level Subcarrier");                                                     
        break;
        
        
        case RSSFC_VAL_ATV_SOUND_SPEC_SUBCAR1:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_SUBCAR_1,
                                                    nomFreqDev), 3, "Nom. Freq. Dev."); 
            
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_SUBCARRIER_1,
                                                    subcarrier), 4, "Subcarrier State");
                                                    
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_FREQ_SUBCARRIER_1,
                                                    frequencySubcarrier), 5, "Frequency Subcarrier");
        
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_LEVEL_SUBCARRIER_1,
                                                    levelSubcarrier), 6, "Level Subcarrier");     
        break;
        
        
        case RSSFC_VAL_ATV_SOUND_SPEC_SUBCAR2:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_NOM_FREQ_DEV_SUBCAR_2,
                                                    nomFreqDev), 3, "Nom. Freq. Dev."); 
            
            viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_SUBCARRIER_2,
                                                    subcarrier), 4, "Subcarrier State");
                                                    
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_FREQ_SUBCARRIER_2,
                                                    frequencySubcarrier), 5, "Frequency Subcarrier");
        
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_LEVEL_SUBCARRIER_2,
                                                    levelSubcarrier), 6, "Level Subcarrier");     
        break;
    }           
  
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

  
/*============================================================================/
 * Function: Configure Analog TV Special Settings
 * Purpose:  This function configures special Analog TV coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvSpecialSettings (ViSession instrSession,
                                                     ViBoolean VSBFilter,
                                                     ViBoolean GDPrecorrection,
                                                     ViReal64 residualCarrier)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_VSB_FILTER, VSBFilter),
                         2, "VSB Filter");
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_GD_PRECORRECTION, GDPrecorrection),
                         3, "GD Precorrection");                         
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_RESIDUAL_CARRIER, residualCarrier),
                         4, "Residual Carrier");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
     
/*============================================================================/
 * Function: Configure Analog TV Special Video and Carrier
 * Purpose:  This function configures special Analog TV coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvSpecialVideoCarrier (ViSession instrSession,
                                                  ViBoolean video,
                                                  ViBoolean carrier)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_VIDEO, video),
                         2, "Video");
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_CARRIER, carrier),
                         3, "Carrier");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
       
/*============================================================================/
 * Function: Configure Analog TV Special VSB Characteristic
 * Purpose:  This function configures the vestigial sideband characteristic
 *           of the respective standard.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvSpecialVsbCharacteristic
             (ViSession instrSession, ViInt32 VSBCharacteristic)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_VSB_CHARACTERISTIC,
                            VSBCharacteristic), 2, "VSB Characteristic");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set Analog TV Special AM Modulation Depth
 * Purpose:  This function sets the modulation depth of the L standard
 *           amplitude-modulated sound.
/============================================================================*/
ViStatus _VI_FUNC rssfc_setAnalogTvSpecialAmModulationDepth
             (ViSession instrSession, ViInt32 AMModulationDepth)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATV_SPECIAL_AM_MODULATION_DEPTH,
                            AMModulationDepth), 2, "AM Modulation Depth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Analog TV Special NICAM
 * Purpose:  This function configures special NICAM parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confAnalogTvSpecialNICAM (ViSession instrSession,
                                                 ViBoolean preemphasis,
                                                 ViInt32 rollOff,
                                                 ViReal64 gainAdjustAF1,
                                                 ViReal64 gainAdjustAF2)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ANALOG_TV_SPECIAL_NICAM_PREEMPHASIS, preemphasis),
                         2, "Preemphasis");                         
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ANALOG_TV_SPECIAL_NICAM_ROLL_OFF, rollOff),
                         3, "Roll Off");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_SPECIAL_NICAM_GAIN_ADJUST_1,
                        gainAdjustAF1), 4, "Gain Adjust AF1");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ANALOG_TV_SPECIAL_NICAM_GAIN_ADJUST_2,
                        gainAdjustAF2), 5, "Gain Adjust AF2");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
    

/*****************************************************************************
 *--------------------------------- Directv ---------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure Directv Input Signal
 * Purpose:  This function configures Directv Input Signal parameters.
/============================================================================*/

ViStatus _VI_FUNC rssfc_confDirectvInputSignal (ViSession instrSession,
                                                ViInt32 source, ViInt32 input,
                                                ViInt32 stuffing,
                                                ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
                                                              
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_SOUR, source),
                         2, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT)  
    	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_INP, input),
                         3, "Input");
	if (source != RSSFC_IQC_SIG_SOUR_TEST) 	
    	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_STUF, stuffing),
                         4, "Stuffing");
    if (source == RSSFC_IQC_SIG_SOUR_TEST)                     
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_TEST, testSignal),
                         5, "Test Signal");      
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Directv Packed Length
 * Purpose:  This function reads Directv packet length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDirectvPackedLength (ViSession instrSession,
                                                 ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Directv Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readDirectvValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     int_result = 0; 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFC_VAL_REQ_SOURCE_DRATE) || (value > RSSFC_VAL_MEAS_TS_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_REQ_SOURCE_DRATE:
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_REQ_SOURCE_DATA_RATE,
                                                    &int_result), 3, "Result"); 
            *result = (ViReal64)int_result;                                                    
        break;
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DIR_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DIR_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DIR_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;

     }           
     
Error: 
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Directv Coding
 * Purpose:  This function configures Directv coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDirectvCoding (ViSession instrSession,
                                           ViReal64 symbolRate,
                                           ViInt32 constellation, 
                                           ViInt32 rollOff,
                                           ViInt32 codeRate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_IQC_DIR_SYMB, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_CONS, constellation),
                         3, "Constellation");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_ROLL, rollOff),
                         4, "Roll Off");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_RATE, codeRate),
                         5, "Code Rate");                         
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Directv Settings
 * Purpose:  This function configures Directv coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confDirectvSettings (ViSession instrSession,
                                             ViInt32 testTSPacket,
                                             ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_IQC_DIR_TSP,
                            testTSPacket), 2, "Test TS Packet");
    viCheckParm (rssfc_SetAttributeViInt32(instrSession, "", RSSFC_ATTR_IQC_DIR_PRBS,
                            PRBS), 3, "PRBS");                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

 


/*****************************************************************************
 *----------------------------- CMMB ----------------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Configure CMMB Input Signal
 * Purpose:  This function configures CMMB Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confCMMBInputSignal(ViSession   instrSession,
                                            ViInt32 source,
                                            ViInt32 input,
                                            ViInt32 stuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_INPUT_SIGNAL_SOURCE, source),
                         2, "Source");
    if (source == RSSFC_VAL_IQC_CMMB_SOUR_EXT)
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_INPUT_SIGNAL_INPUT, input),
                         3, "Input");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_CMMB_INPUT_SIGNAL_STUFFING, (ViBoolean)stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure CMMB Test Signal
 * Purpose:  This function sets the signal type for the test signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_configureCMMBTestSignal(ViSession instrSession,
                                           ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         2, "Test Signal"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read CMMB Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readCMMBValues(ViSession instrSession,
                                        ViInt32 value,
                                        ViReal64* result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_MEAS_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){                                                       
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_CMMB_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_CMMB_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Set CMMB Number Of Services
 * Purpose:  This function sets the number of services.
/============================================================================*/
ViStatus _VI_FUNC rssfc_SetCMMBNumberOfServices(ViSession   instrSession,
                                            ViInt32 numberOfServices)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_INPUT_SIGNAL_NUMBER_OF_SERVICES, numberOfServices),
                         2, "Number Of Services");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*============================================================================/
 * Function: Configure CMMB Coding
 * Purpose:  This function configures CMMB coding parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confCMMBCoding (ViSession instrSession,
                                        ViInt32 control, ViInt32 bandwidth) 

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_CODING_CONTROL, control),
                         2, "Control");
    
    if (control == RSSFC_VAL_IQC_CMMB_CONTROL_MANUAL)
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_CODING_BANDWIDTH, bandwidth),
                         3, "Bandwidth");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                         

/*============================================================================/
 * Function: Read CMMB OFDM Mode
 * Purpose:  This function returns number of carriers of the CMMB coder.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadCMMBOFDMMode (ViSession instrSession, ViInt32* OFDMMode)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    if (!OFDMMode) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "OFDM Mode");
        
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_CODING_OFDM_MODE,
                                            OFDMMode), 2, "OFDM Mode");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure CMMB PLCH Table
 * Purpose:  This function configures CMMB coding PLCH table parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confCMMBPLCHTable (ViSession instrSession, ViInt32 PLCH,
                                           ViInt32 RS, ViInt32 byteIL, ViInt32 LDPC,
                                           ViInt32 constel, ViInt32 SCRMode,
                                           ViInt32 startTS, ViInt32 stopTS)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
   
    if ((PLCH < 0) || (PLCH > 39))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLCH");
    
    if ((RS < RSSFC_VAL_IQC_CMMB_RS_0) || (RS > RSSFC_VAL_IQC_CMMB_RS_3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "RS");
        
    if ((byteIL < 1) || (byteIL > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Byte IL");     
    
    if ((LDPC < RSSFC_VAL_IQC_CMMB_LDPC_R1_2) || (LDPC > RSSFC_VAL_IQC_CMMB_LDPC_R3_4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "LDPC");     

    if ((constel < RSSFC_VAL_IQC_CMMB_CONST_C2) || (constel > RSSFC_VAL_IQC_CMMB_CONST_C16))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "Constel");   
    
    if ((SCRMode < 0) || (SCRMode > 7))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 7, "SCR Mode");
    
    if ((startTS < 0) || (startTS > 39))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 8, "Start TS");  
        
    if ((stopTS < 0) || (stopTS > 39))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 9, "Stop TS");  
        
    sprintf (buffer,":CMMB:RS %ld,%s;BYIM %ld,%ld;LDPC %ld,%s;CONS %ld,%s;SCRM %ld,%ld;STAR %ld,%ld;STOP %ld,%ld",
        PLCH,cmmbPlchRsArr[RS],PLCH,byteIL,
        PLCH,cmmbPlchLdpdArr[LDPC],
        PLCH,cmmbPlchConstArr[constel],PLCH,SCRMode,
        PLCH,startTS,PLCH,stopTS);
  
    viCheckErr( viPrintf (instrSession, "%s", buffer));  
    }   
    
    viCheckErr(rssfc_CheckStatus (instrSession));    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 
  
/*============================================================================/
 * Function: Configure CMMB Settings
 * Purpose:  This function configures CMMB coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confCMMBSettings (ViSession instrSession,
                                          ViInt32 transmitterID, ViInt32 areaID,
                                          ViInt32 payloadTestSignal, ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_SETTINGS_TRANSMITTER_ID, transmitterID),
                         2, "Transmitter ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_SETTINGS_AREA_ID, areaID),
                         3, "Area ID"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_SETTINGS_PAYLOAD_TEST_SIGNAL, payloadTestSignal),
                         4, "Payload Test Signal"); 
    switch (payloadTestSignal){
        case  RSSFC_VAL_IQC_PRBS:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CMMB_SETTINGS_PRBS, PRBS),
                         5, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *--------------------------- ATSC-M/H  -------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure ATSM Input Signal
 * Purpose:  This function configures ATSM Input Signal parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confATSMInputSignal (ViSession instrSession,
                                             ViInt32 source, ViInt32 input,
                                             ViBoolean stuffing)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
                                                              
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_SOURCE, source),
                         2, "Source");
    if (source == RSSFC_IQC_SIG_SOUR_EXT)
		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_INPUT, input),
                         3, "Input"); 
    if (source != RSSFC_IQC_SIG_SOUR_TEST)
		viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATSM_STUFFING, stuffing),
                         4, "Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}   

/*============================================================================/
 * Function: Configure ATSM Test Signal
 * Purpose:  This function configures ATSM Test Signal.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confATSMTestSignal (ViSession instrSession,
                                            ViInt32 testSignal)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));      
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_TEST_SIGNAL, testSignal),
                         2, "Test Signal");   
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read ATSM Packed Length
 * Purpose:  This function reads ATSM packet length.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readATSMPackedLength (ViSession instrSession,
                                              ViInt32 *packedLength)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_READ_ATSM_PACKET_LENGTH,
                                            packedLength), 2, "Packed Length");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Read ATSM Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readATSMValues (ViSession instrSession,
                                        ViInt32 value, 
                                        ViReal64 *result)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    
    if ((value < RSSFC_VAL_MAX_USEFUL_DRATE) || (value > RSSFC_VAL_USEFUL_DRATE))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
        
    switch (value){
        case RSSFC_VAL_MAX_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_ATSM_MAX_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_ATSM_MEASURED_DATA_RATE,
                                                     result), 3, "Result");
        break;
        case RSSFC_VAL_USEFUL_DRATE:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_READ_ATSM_USEFUL_DATA_RATE,
                                                     result), 3, "Result");
        break;
	
     }           
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read ATSM Values
 * Purpose:  This function reads selected coding value.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confATSMCoding (ViSession instrSession,
                                        ViReal64 symbolRate, 
                                        ViBoolean mHState)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATSM_SYMBOL_RATE, symbolRate),
                         2, "Symbol Rate"); 
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATSM_MH_STATE, mHState),
                         3, "M/H State");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read ATSM Constellation
 * Purpose:  This function queries the VSB constellation firmly set to 8VSB.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadATSMConstellation (ViSession instrSession,
                                               ViInt32 *constellation)
{
   ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_CONSTELLATION,
                                            constellation), 2, "Constellation");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read ATSM Roll Off
 * Purpose:  This function queries queries the roll off.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ReadATSMRollOff (ViSession instrSession,
                                         ViReal64 *rollOff)
{
   ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_ATSM_ROLL_OFF,
                                            rollOff), 2, "Roll Off");
   Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSM Settings
 * Purpose:  This function configures ATSM coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confATSMSettings (ViSession instrSession,
                                          ViInt32 MHEPID, 
                                          ViBoolean RFWatermark,
                                          ViBoolean MTXIDTransmission,
                                          ViInt32 testTSPacket,
                                          ViInt32 payloadTest, 
                                          ViInt32 PRBS)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_MHE_PID, MHEPID),
                         2, "MHE PID"); 
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATSM_WATERMARK, RFWatermark),
                         3, "RF Watermark");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ATSM_MTXID_TRANSMISSION, MTXIDTransmission),
                         4, "MTXID Transmission");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_TEST_TS_PACKET, testTSPacket),
                         5, "Test TS Packet"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_PAYLOAD_TEST, payloadTest),
                         6, "Payload Test");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_PRBS, PRBS),
                         7, "PRBS");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSM Watermark
 * Purpose:  This function configures ATSM watermark. The ATSC A/110B standard 
 *           specifies that the RF signal can be overlaid with a watermark. 
 *           For this purpose, the modulator calculates a specific Kasami 
 *           sequence from the TX address and network ID. This sequence first
 *           modulates BPSK and is then added to the payload signal as buried
 *           spread spectrum. Thus, a unique watermark is provided for each
 *           transmitter.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confATSMWatermark (ViSession instrSession,
                                           ViInt32 TXAddress, 
                                           ViInt32 networkID,
                                           ViInt32 buryRatio)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_TX_ADDRESS, TXAddress),
                         2, "TX Address"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_NETWORK_ID, networkID),
                         3, "Network ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_BURY_RATIO, buryRatio),
                         4, "Bury Ratio");
   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure ATSM MTXID Transmission
 * Purpose:  This function configures ATSM MTXID Transmission. 
 *           The transmission of the MTXID in the FRAME SYNC signal can be 
 *           described as a technique for detecting the respective transmitter 
 *           that is complementary to the RF watermark technique. The MTXID 
 *           comprises the market ID and transmitter ID.
 *           The main advantage of this method is that the error vector of
 *           the useful signal is not enlarged. Furthermore, the information
 *           transmitted in this case is to be used for highly accurate
 *           position-finding.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confATSMMTXIDTransmission (ViSession instrSession,
                                                   ViInt32 transmitterID,
                                                   ViInt32 marketID)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_TRANSMITTER_ID, transmitterID),
                         2, "Transmitter ID"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ATSM_MARKET_ID, marketID),
                         3, "Market ID");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *------------------------------ DVB-C2 -------------------------------------*
 *****************************************************************************/

/*===========================================================================*/
/* Function: Configure DVBC2 Input Signal
/* Purpose:  This function configures DVB-C2 input signal parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configureDVBC2InputSignal (ViSession instrSession, ViInt32 plp,
                                           ViInt32 inputFormat, ViInt32 source,
                                           ViInt32 input)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]);  
    	
   	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_INPUT_SIGNAL_INPUT_FORMAT, inputFormat),
                         3, "Input Format");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_INPUT_SIGNAL_SOURCE, source),
                         4, "Source");
	if (source == RSSFC_IQC_SIG_SOUR_EXT)   
		viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_INPUT_SIGNAL_INPUT, input),
                         5, "Input");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 	

/*===========================================================================*/
/* Function: Configure DVBC2 Number Of PLP
/* Purpose:  This function configures DVB-T2 physical layer pipe (PLP)
/*           number.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2NumberOfPLP (ViSession instrSession,
                                                   ViInt32 numberOfPLP)
{
    ViStatus    error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_INPUT_SIGNAL_NUMBER_OF_PLP, numberOfPLP),
                         2, "Number Of PLP");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Read DVBT2 Values
/* Purpose:  This function reads selected coding value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2Values(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	value,
	ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]); 

	switch (value){
        case RSSFC_VAL_MAX_USEFUL_DRATE:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_READ_DVBC2_INPUT_SIGNAL_MAX_DR, 
	            result), 4, "Result");
        break;
        case RSSFC_VAL_MEAS_DRATE:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_READ_DVBC2_INPUT_SIGNAL_MEAS_DR, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_USEFUL_DRATE:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_READ_DVBC2_INPUT_SIGNAL_USEFUL_DR, 
                result), 4, "Result");
        break;
		
		
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
	}

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Test Signal
/* Purpose:  This function configures DVB-C2 Test Signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVB2CTestSignal(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32	testSignal
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]); 

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_INPUT_SIGNAL_TEST_SIGNAL, testSignal),
                         3, "Test Signal");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Configure DVBC2 Mode Stream Adaptation
/* Purpose:  This function configures DVB-C2 Mode Stream Adaptation
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2ModeStreamAdaptation
             (ViSession instrSession, ViInt32 plp, ViInt32 PLPID,
              ViInt32 BBMode, ViInt32 ISSY,
              ViBoolean nullPacketDel)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

	sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]); 
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_MODE_STREAM_ADAPT_PLP_ID, PLPID),
                         3, "PLP ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_MODE_STREAM_ADAPT_BB_MODE, BBMode),
                         4, "BB Mode");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_MODE_STREAM_ADAPT_ISSY, ISSY),
                         5, "ISSY");
	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, buf1, RSSFC_ATTR_DVBC2_MODE_STREAM_ADAPT_NULL_PACKET_DEL, nullPacketDel),
                         6, "Null Packet Del"); 
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*===========================================================================*/
/* Function: Read DVBC2 Mode Stream Adaptation PLP Type
/* Purpose:  This function queries the PLP type.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2ModeStreamAdaptationPLPType(
	ViSession	instrSession,
	ViInt32	plp,
	ViInt32*	plpType
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
	 if (!plpType) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "PLP Type");   
	 
	sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]);

    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_MODE_STREAM_ADAPT_PLP_TYPE, plpType),
                         3, "PLP Type");                        
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;

}

/*===========================================================================*/
/* Function: Configure DVBC2 BICM PLP Mapping Parameters
/* Purpose:  This function configures DVB-C2 Bit Interleaved Coding and PLP
/*  	     Mapping parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2BICMPLPMappingParameters
             (ViSession instrSession, ViInt32 plp, ViInt32 FECFrame,
              ViInt32 codeRate, ViInt32 constellation, ViInt32 mapToDSlice)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

	sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]); 
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_BICM_FEC_FRAME, FECFrame),
                         3, "FEC Frame");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_BICM_CODE_RATE, codeRate),
                         4, "Code Rate");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_BICM_CONSTELL, constellation),
                         5, "Constellation");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_BICM_MAP_TO_DSLICE, mapToDSlice),
                         6, "Map To DSlice");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBC2 BICM XFEC Frame Length
/* Purpose:  This function queries the length of an XFECFrame expressed as
/*           the number of OFDM cells.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2BICMXFECFrameLength (ViSession instrSession,
                                                      ViInt32 plp,
                                                      ViInt32* XFECFrameLength)

{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((plp < 1) || (plp > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "PLP"); 

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	sprintf (buf1, "PLP%s", dvbC2PlpArr[plp]);   
	
	if (!XFECFrameLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "XFEC Frame Length");
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_BICM_XFEC_FRAME_LENGTH, 
               XFECFrameLength), 3, "XFEC Frame Length");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Min Max
/* Purpose:  This function returns the minimum and maximum carrier index for
/*           the C2 signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2DataSliceMinMax (ViSession instrSession,
                                                  ViInt32* min_REL_DxTu,
                                                  ViInt32* max_REL_DxTu,
                                                  ViInt32* min_ABS_DxTu,
                                                  ViInt32* max_ABS_DxTu)
{
 ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));

	if (!min_REL_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Min REL Dx/Tu");
	if (!max_REL_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Max REL Dx/Tu"); 
	if (!min_ABS_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 4, "Min ABS Dx/Tu"); 
	if (!max_ABS_DxTu) viCheckParm (RS_ERROR_INVALID_PARAMETER, 5, "Max ABS Dx/Tu"); 
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "MIN", RSSFC_ATTR_DVBC2_DATA_SLICE_MIN_MAX_REL, 
               min_REL_DxTu), 2, "Min REL Dx/Tu");
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "MAX", RSSFC_ATTR_DVBC2_DATA_SLICE_MIN_MAX_REL, 
               max_REL_DxTu), 3, "Max REL Dx/Tu");
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "MIN", RSSFC_ATTR_DVBC2_DATA_SLICE_MIN_MAX_ABS, 
               min_ABS_DxTu), 4, "Min ABS Dx/Tu");
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "MAX", RSSFC_ATTR_DVBC2_DATA_SLICE_MIN_MAX_ABS, 
               max_ABS_DxTu), 5, "Max ABS Dx/Tu");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Number Of Data Slices
/* Purpose:  This function configures number of data slices.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2NumberOfDataSlices (ViSession instrSession,
                                                          ViInt32 numberOfDataSlices)
{
    ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_DATA_SLICE_NUMBER_OF_DATA_SLICES,
		numberOfDataSlices), 2, "Number Of Data Slices");
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Configure DVBC2 Data Slice Parameters
/* Purpose:  This function configures DVB-C2 Data Slice parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2DataSliceParameters
             (ViSession instrSession, ViInt32 DSlice, ViInt32 DSliceID_HEX,
              ViInt32 DSliceTIDepth, ViInt32 DSliceType, ViInt32 FECHeaderType,
              ViInt32 numXFECFrames)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice"); 

	sprintf (buf1, "DSL%s", dSliceArr[DSlice]); 
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_DSLICE_ID, DSliceID_HEX),
                         3, "DS liceID HEX");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_DSLICE_TI_DEPTH, DSliceTIDepth),
                         4, "DSlice TI Depth");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_DSLICE_TYPE, DSliceType),
                         5, "DSlice Type");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_FEC_HEADER_TYPE, FECHeaderType),
                         6, "FEC Header Type");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_NUM_XFEC_FRAMES, numXFECFrames),
                         7, "Num XFEC Frames");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Num PLP
/* Purpose:  This function returns the number of PLPs assigned to this data
/*           slice. PLPs are assigned to data slices under MAP TO DSLICE.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2DataSliceNumPLP (ViSession instrSession,
                                                  ViInt32 DSlice,
                                                  ViInt32* DSliceNumPLP)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice"); 

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	sprintf (buf1, "DSL%s", dSliceArr[DSlice]);   
	
	if (!DSliceNumPLP) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "DSlice Num PLP");
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_DSLICE_NUM_PLP, 
               DSliceNumPLP), 3, "DSlice Num PLP");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Current Data Slice Parameters
/* Purpose:  This function configures DVB-C2 Current Data Slice parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2CurrentDataSliceParameters
             (ViSession instrSession, ViInt32 DSlice, ViInt32 offsetLeftDxTu,
              ViInt32 tunePos_REL_DxTu, ViInt32 offsetRightDxTu)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice"); 

	sprintf (buf1, "DSL%s", dSliceArr[DSlice]);  
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_OFFSET_LEFT, offsetLeftDxTu),
                         3, "Offset Left Dx/Tu");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_TUNE_POS_REL, tunePos_REL_DxTu),
                         4, "Tune Pos REL Dx/Tu");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_OFFSET_RIGHT, offsetRightDxTu),
                         5, "Offset Right Dx/Tu");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2DataSliceValues(
	ViSession	instrSession,
	ViInt32	DSlice,
	ViInt32	value,
	ViInt32* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "DSL%s", dSliceArr[DSlice]); 

	switch (value){
        case RSSFC_VAL_DVBC2_DSLICE_START_REL:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_START_REL, 
	            result), 4, "Result");
        break;
        case RSSFC_VAL_DVBC2_DSLICE_STOP_REL:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_STOP_REL, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_WIDTH:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_WIDTH, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_START_ABS:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_START_ABS, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_TUNE_POS_ABS:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_TUNE_POS_ABS, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_STOP_ABS:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_STOP_ABS, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_TUNE_POS_REL:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_TUNE_POS_REL, 
                result), 4, "Result");
        break;
		
		
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Value"); 
	}

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Data Slice Frequency Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2DataSliceFrequencyValues(
	ViSession	instrSession,
	ViInt32	DSlice,
	ViInt32	DSliceValue,
	ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((DSlice < 0) || (DSlice > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "DSlice");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "DSL%s", dSliceArr[DSlice]);  

	switch (DSliceValue){
       	case RSSFC_VAL_DVBC2_DSLICE_TUNE_FREQ:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_TUNE_FREQ, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_START_FREQ:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_START_FREQ, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_DSLICE_STOP_FREQ:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBC2_DATA_SLICE_STOP_FREQ, 
                result), 4, "Result");
        break;
		
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "DSlice Value"); 
	}

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Number Of Notches
/* Purpose:  This function configures Number of notches.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2NumberOfNotches (ViSession instrSession,
                                                  ViInt32 numberOfNotches)
{
 ViStatus    error = VI_SUCCESS;

   
	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_NOTCH_NUMBER_OF_NOTCHES, 
               numberOfNotches), 3, "Number Of Notches");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Number Of Notches
/* Purpose:  This function returns Number of notches.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2NumberOfNotches (ViSession instrSession,
                                                  ViInt32* numberOfNotches)
{
 ViStatus    error = VI_SUCCESS;

   
	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (!numberOfNotches) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Number Of Notches");
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_NOTCH_NUMBER_OF_NOTCHES, 
               numberOfNotches), 3, "Number Of Notches");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Notch Type
/* Purpose:  This function queries the Notch type.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2NotchType (ViSession instrSession,
                                            ViInt32 notch, ViInt32* notchType)

{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = ""; 
	 
	checkErr( Rs_LockSession (instrSession, VI_NULL));        
	
    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");
	
    checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (!notchType) viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Notch Type");
	
	sprintf (buf1, "N%s", notchArr[notch]); 
     
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_TYPE, notchType),
                         3, "Notch Type");
  	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Notch Values
/* Purpose:  This function selects and sets the relevant notch parameters in
/*           the work view.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2NotchValues (ViSession instrSession,
                                                   ViInt32 notch,
                                                   ViInt32 start_REL_DxTu,
                                                   ViInt32 stop_REL_DxTu,
                                                   ViInt32 width_REL_DxTu)
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = ""; 
	 
	checkErr( Rs_LockSession (instrSession, VI_NULL));        
	
    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "N%s", notchArr[notch]);
     
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_START_REL, start_REL_DxTu),
                         3, "Start REL [Dx/Tu]");
	viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_STOP_REL, stop_REL_DxTu),
                         4, "Stop REL [Dx/Tu]");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_WIDTH, width_REL_DxTu),
                         5, "Width REL [Dx/Tu]");		
  	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Notch Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2NotchValues(
	ViSession	instrSession,
	ViInt32	notch,
	ViInt32	notchParameter,
	ViInt32* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");

    checkErr( Rs_LockSession (instrSession, VI_NULL));

	sprintf (buf1, "N%s", notchArr[notch]);
	
	switch (notchParameter){
        case RSSFC_VAL_DVBC2_NOTCH_START_REL:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_START_REL, 
	            result), 4, "Result");
        break;
        case RSSFC_VAL_DVBC2_NOTCH_STOP_REL:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_STOP_REL, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_NOTCH_WIDTH:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_WIDTH, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_NOTCH_START_ABS:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_START_ABS, 
                result), 4, "Result");
        break;
		case RSSFC_VAL_DVBC2_NOTCH_STOP_ABS:
    		viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_STOP_ABS, 
                result), 4, "Result");
        break;
		
		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Notch Parameter"); 

	}

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Notch Frequency Values
/* Purpose:  This function reads selected data slice value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2NotchFrequencyValues(
	ViSession	instrSession,
	ViInt32	notch,
	ViInt32	notchParameter,
	ViReal64* result
)
{
 ViStatus    error = VI_SUCCESS;
 ViChar      buf1[RSSFC_TOKEN_BUFFER_SIZE] = "";

    if ((notch < 0) || (notch > 3))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Notch");

   checkErr( Rs_LockSession (instrSession, VI_NULL));
   
	sprintf (buf1, "N%s", notchArr[notch]);
	
	switch (notchParameter){
        case RSSFC_VAL_DVBC2_NOTCH_START_FREQ:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_START_FREQ, 
	            result), 4, "Result");
        break;
        case RSSFC_VAL_DVBC2_NOTCH_STOP_FREQ:
    		viCheckParm (rssfc_GetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_DVBC2_NOTCH_STOP_FREQ, 
                result), 4, "Result");
        break;

		default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Notch Parameter"); 

	}

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Framing OFDM Parameters
/* Purpose:  This function configures FRAMING + OFDM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2FramingOFDMParameters
             (ViSession instrSession, ViInt32 channelBandwidth,
              ViInt32 numBundledCH, ViInt32 guardInterval)
{
    ViStatus    error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

     
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_CHANNEL_BANDWIDTH, channelBandwidth),
                         2, "Channel Bandwidth");
   
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_GUARD_INTERVAL, guardInterval),
                         4, "Guard Interval");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 C2 Bandwidth
/* Purpose:  This function configures the total bandwidth of the C2 signal.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2FramingOFDMC2Bandwidth
             (ViSession instrSession, ViInt32 bandwidth)
{
    ViStatus    error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

     
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_C2_BANDWIDTH, bandwidth),
                         2, "C2 Bandwidth");
  	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
/*===========================================================================*/
/* Function: Read DVBC2 Framing OFDM Used Bandwidth
/* Purpose:  This function queries the used channel bandwidth.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2FramingOFDMUsedBandwidth
             (ViSession instrSession, ViReal64* usedBandwidth
)
{
	ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (!usedBandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Used Bandwidth");
															   
	viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_USED_BANDWIDTH, 
               usedBandwidth), 2, "Used Bandwidth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBC2 Framing OFDM Baseband Filter
/* Purpose:  This function configures the type of baseband filtering.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2FramingOFDMBasebandFilter (ViSession instrSession,
                                                  ViInt32 basebandFilter)
{
 ViStatus    error = VI_SUCCESS;

   
	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_BASEBAND_FILTER,
                            basebandFilter), 2, "Baseband Filter");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure DVBC2 Framing OFDM Adjust RF Frequency
/* Purpose:  This function sets the RF frequency to the calculated center
 *           frequency value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2FramingOFDMAdjustRFFrequency (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_ADJUST_RF_FREQUENCY, "")); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read DVBC2 Framing OFDM Calculate Center Frequency
/* Purpose:  This function queries the calculated center frequency.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2FramingOFDMCalculateCenterFrequency
             (ViSession instrSession, ViReal64* calcCenterFrequency
)
{
	ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));
	
	if (!calcCenterFrequency) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Calculate Center Frequency");
															   
	viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_CALC_CENTER_FREQUENCY, 
               calcCenterFrequency), 2, "Calculate Center Frequency");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Read DVBC2 Framing OFDM Parameters
/* Purpose:  This function queries the OFDM symbols parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadDVBC2FramingOFDMParameters (ViSession instrSession,
	                                                    ViInt32* DX,
                                                        ViReal64* carrierSpacing_1Tu,
                                                        ViInt32* numberOfDataSymbolC2Frame,
                                                        ViInt32* numberOfPreambleSymbols,													
														ViReal64* startFrequency,
														ViReal64* calcCenterFrequency)
{
	ViStatus    error = VI_SUCCESS;

	checkErr( Rs_LockSession (instrSession, VI_NULL));

	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_DX, 
               DX), 2, "DX");

	viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_CARRIER_SPACING, 
               carrierSpacing_1Tu), 3, "Carrier Spacing 1/Tu");	
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_NUMBER_OF_DATA_SYMBOLS_PER_C2_FRAME, 
               numberOfDataSymbolC2Frame), 4, "Number Of Data Symbol C2 Frame");
	
	viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_NUMBER_OF_PREAMBLE_SYMBOLS, 
               numberOfPreambleSymbols), 5, "Number Of Preamble Symbols");

	viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_START_FREQUENCY, 
               startFrequency), 6, "Start Frequency");
	
	viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_DVBC2_FRAMING_CALC_CENTER_FREQUENCY, 
               calcCenterFrequency), 7, "Calc Center Frequency");			
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure DVBC2 C2 System Parameters
/* Purpose:  This function configures C2 system identification parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2C2SystemParameters (ViSession instrSession,
                                                          ViInt32 networkID,
                                                          ViInt32 c2SystemID,
                                                          ViInt32 l1TimeInterleaver,
                                                          ViBoolean systemReservedTones)
{
    ViStatus    error = VI_SUCCESS;
	
	checkErr( Rs_LockSession (instrSession, VI_NULL));        

     
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_C2_SYSTEM_NETWORK_ID, networkID),
                         2, "Network ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_C2_SYSTEM_ID, c2SystemID),
                         3, "C2 System ID");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_C2_SYSTEM_L1_TIME_INTERLEAVER, l1TimeInterleaver),
                         4, "L1 Time Interleaver");
	viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_DVBC2_C2_SYSTEM_RESERVED_TONES, systemReservedTones),
                         5, "System Reserved Tones");
	
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}	

/*============================================================================/
 * Function: Configure DVBC2 Settings
 * Purpose:  This function configures DVB-C2 coding settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureDVBC2Settings (ViSession instrSession,
                                           ViInt32 testTSPacket,
                                           ViInt32 payloadTestTSPacket, ViInt32 prbs)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    		  

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_SETTINGS_TEST_TS_PACKET, testTSPacket),
		                 2, "Test TS Packet");  
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_SETTINGS_PAYLOAD_TEST_TS_PACKET, payloadTestTSPacket),
                         3, "Payload Test TS Packet");      
    switch (payloadTestTSPacket){
        case  RSSFC_VAL_IQC_PRBS:
            viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_DVBC2_SETTINGS_PRBS, prbs),
                         4, "PRBS"); 
    }   
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*****************************************************************************
 *----------------------- Audio Broadcasting --------------------------------*
 *****************************************************************************/

/*===========================================================================*/
/* Function: Configure Audio BC FM Input Signal
/* Purpose:  This function configures Audio Broadcasting FM Input Signal 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configAudioBCFMInputSignal(
    ViSession   instrSession,
    ViInt32 sourceLeft,
    ViInt32 sourceRight,
    ViInt32 afMode
)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SOURCE_LEFT, sourceLeft),
                         2, "Source Left"); 
    if (sourceLeft != RSSFC_VAL_AUDIO_BC_INPUT_MPX){
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SOURCE_RIGHT, sourceRight),
                         3, "Source Right");
        viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AF_MODE, afMode),
                         4, "AF Mode");
    }

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC FM Source
/* Purpose:  This function configures Audio BC FM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confAudioBCFMSource(
    ViSession   instrSession,
    ViInt32 source,
    ViReal64    frequency,
    ViReal64    level
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (source){
        case RSSFC_VAL_AUDIO_BC_SOUR_LEFT:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SOURCE_FREQUENCY_LEFT, 
                        frequency), 3, "Frequency");
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SOURCE_LEVEL_LEFT, 
                        level), 4, "Level");
        break;
        case RSSFC_VAL_AUDIO_BC_SOUR_RIGHT:
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SOURCE_FREQUENCY_RIGHT, 
                        frequency), 3, "Frequency");
            viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SOURCE_LEVEL_RIGHT, 
                        level), 4, "Level");
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Source");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC FM RDS RBDS
/* Purpose:  This function configures Audio Broadcasting FM RDS/RBDS
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configAudioBCFMRDSRBDS (ViSession instrSession,
                                                ViString groupSequence, ViInt32 RS_PI,
                                                ViString PS, ViBoolean TP,
                                                ViBoolean TA, ViInt32 PTY,
                                                ViString PTYN, ViString RT,
                                                ViInt32 MS)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_GROUP_SEQUENCE, groupSequence),
                         2, "Group Sequence"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_PI, RS_PI),
                         3, "Program Identification"); 
    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_PS, PS),
                         4, "PS"); 
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_TP, TP),
                         5, "TP");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_TA, TA),
                         6, "TA");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_PTY, PTY),
                         7, "PTY");
    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_PTYN, PTYN),
                         8, "PTYN"); 
    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_RT, RT),
                         9, "RT"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_MS, MS),
                         10, "MS");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Read Audio BC FM RDS RBDS Radio Text Status
/* Purpose:  This function queries the data transfer status of the radio text RT.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadAudioBCFMRDSRBDSRadioTextStatus(
    ViSession   instrSession,
    ViInt32*    rtReady
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!rtReady) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "RT Ready");
    
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_RT_READY, 
               rtReady), 2, "RT Ready");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure Audio BC FM RDS RBDS Decoder Identification
/* Purpose:  This function configures Audio Broadcasting FM RDS/RBDS Decoder
/*           Identification parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configAudioBCFMRDSRBDSDecoderIdn
             (ViSession instrSession, ViBoolean DIDynamicPTY,
              ViBoolean DICompressed, ViBoolean DIArtHead, ViBoolean DIStereo,
              ViBoolean CT, ViString CTOffset, ViInt32 AFMethod)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_DYNAMIC_PTY, DIDynamicPTY),
                         2, "DI Dynamic PTY");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_COMPRESSED, DICompressed),
                         3, "DI Compressed");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_ART_HEAD, DIArtHead),
                         4, "DI Art Head");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_STEREO, DIStereo),
                         5, "DI Stereo");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_CT, CT),
                         6, "CT");
    if (CT)
        viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_CT_OFFSET, CTOffset),
                            7, "CT Offset"); 
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_RDS_DI_AF_METHOD, AFMethod),
                         8, "AF Method");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Set Audio BC FM AF A Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. in AF method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCFMAFAFreq (ViSession instrSession, ViInt32 numberOfFreq)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AF_A_NUMBER_OF_FREQ,
                    numberOfFreq), 2, "Number Of Freq");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

 
/*===========================================================================*/
/* Function: Configure Audio BC FM AF A Frequency
/* Purpose:  This function sets the alternative frequencies 1 to 25 in AF
/*           method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCFMAFAFrequency (ViSession instrSession,
                                                        ViInt32 af,
                                                        ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";      

    if ((af < 1) || (af > 25))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "AF"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
      
    sprintf (buf1, "AF%ld", af);

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_AF_A_FREQUENCY, frequency), 
                    3, "Frequency"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Set Audio BC FM AF B Number Of Lists
/* Purpose:  This function sets the NUMBER OF LISTS in AF method B (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCFMAFBList (ViSession instrSession,
                                             ViInt32 numberOfLists)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AF_B_NUMBER_OF_LISTS,
                    numberOfLists), 2, "Number Of Lists");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Set Audio BC FM AF B Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. and TUNING FREQ of a list
/*           in AF method B (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCFMAFBFreq (ViSession instrSession,
                                             ViInt32 list,
                                             ViInt32 numberOfFreq,
                                             ViReal64 tuningFreq)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((list < 1) || (list > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "List"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "L%ld", list); 
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_AF_B_NUMBER_OF_FREQ,
                    numberOfFreq), 3, "Number Of Freq");

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_AF_B_TUNING_FREQ,
                    tuningFreq),  4, "Tuning Freq"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 


/*===========================================================================*/
/* Function: Configure Audio BC FM AF B Frequency
/* Purpose:  This function sets ascending or descendin alternative frequency
/*           of a list in AF method B (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCFMAFBFrequency (ViSession instrSession,
                                                        ViInt32 list, ViInt32 af,
                                                        ViReal64 frequency,
                                                        ViInt32 order)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((list < 1) || (list > 5))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "List"); 
    if ((af < 1) || (af > 12))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "AF"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "L%ld,AF%ld", list, af);
       
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_AF_B_AF_FREQUENCY,
                    frequency),  4, "Frequency"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_AF_B_AF_ORDER,
                    order), 5, "order");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Configure Audio BC FM Audio Modulation
/* Purpose:  This function configures Audio Broadcasting FM EON parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configAudioBCFMEONParameters (ViSession instrSession,
                                                      ViInt32   EONPI,
                                                      ViString  EONPS,
                                                      ViBoolean EONTP,
                                                      ViBoolean EONTA,
                                                      ViBoolean EONLA,
                                                      ViBoolean EONEG,
                                                      ViBoolean EONILS,
                                                      ViInt32 EONLSN,
                                                      ViInt32 EONPTY,
                                                      ViInt32 EONPIN,
                                                      ViInt32 EONAFMethod)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_PI, EONPI),
                            2, "EON PI");
    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_PS, EONPS),
                            3, "EON PS"); 
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_TP, EONTP),
                            4, "EON TP");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_TA, EONTA),
                            5, "EON TA");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_LA, EONLA),
                            6, "EON LA");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_EG, EONEG),
                            7, "EON EG");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_ILS, EONILS),
                            8, "EON ILS");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_LSN, EONLSN),
                            9, "EON LSN");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_PTY, EONPTY),
                            10, "EON PTY");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_PIN, EONPIN),
                            11, "EON PIN");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_AF_METHOD, EONAFMethod),
                            12, "EON AF Method");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}


/*===========================================================================*/
/* Function: Set Audio BC FM EON AF A Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. in EON AF method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCFMEONAFAFreq (ViSession instrSession,
                                                ViInt32 numberOfFreq)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_AF_A_NUMBER_OF_FREQ,
                    numberOfFreq), 2, "Number Of Freq");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

 
/*===========================================================================*/
/* Function: Configure Audio BC FM EON AF A Frequency
/* Purpose:  This function sets one of 25 alternative frequencies in AF
/*           method A (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCFMEONAFAFrequency
             (ViSession instrSession, ViInt32 af, ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((af < 1) || (af > 25))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "AF"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    sprintf (buf1, "AF%ld", af);    

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_EON_AF_A_FREQUENCY, frequency), 
                    3, "Frequency"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Set Audio BC FM AF Mapped Freq Number Of Freq
/* Purpose:  This function sets the NUMBER OF FREQ. and TUNING FREQ of EON,
/*           AF method MAPPED FREQ. (FM)
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCFMEONAFMappedFreqNumberOfFreq
             (ViSession instrSession, ViInt32 numberOfFreq,
              ViReal64 tuningFreq)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_NUMBER_OF_FREQ,
                    numberOfFreq), 2, "Number Of Freq");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_TUNING_FREQ, 
                    tuningFreq), 3, "Tuning Freq");     

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}

/*===========================================================================*/
/* Function: Configure Audio BC FM Audio Modulation
/* Purpose:  This function sets an alternative frequency in EON, AF method
/*           MAPPED FREQ. (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCFMEONAFMappedFrequency
             (ViSession instrSession, ViInt32 af, ViReal64 frequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buf1[RSSFC_IO_BUFFER_SIZE] = "";

    if ((af < 1) || (af > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "AF"); 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buf1, "AF%ld", af);    
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, buf1, RSSFC_ATTR_AUDIO_BC_EON_AF_MAPPED_FREQ_AF_FREQUENCY, frequency), 
                    3, "Frequency"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Enable Audio BC FM TMC
/* Purpose:  This function switches the TMC on or off (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_EnableAudioBCFMTMC (ViSession instrSession,
                                            ViBoolean TMC)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_TMC_TMC, TMC),
                         2, "TMC");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;           
}

/*===========================================================================*/
/* Function: Audio BC FM TMC Send Data
/* Purpose:  This function sends the data of the traffic massege channel TMC (FM)
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_AudioBCFMTMCSendData (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_TMC_APPLY, "")); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set Audio BC TMC Group 3A VAR Parameters
/* Purpose:  This function sets the TMC group 3A VAR 1 or VAR 2 (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCTMCGroup3AVARParameters
             (ViSession instrSession, ViInt32 group3AVAR00,
              ViInt32 group3AVAR01)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_TMC_GROUP_3A_VAR_00,
                            group3AVAR00), 2, "Group 3A VAR 00");
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_TMC_GROUP_3A_VAR_01,
                            group3AVAR01), 3, "Group 3A VAR 01");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set Audio BC TMC Number Of 8A Groups
/* Purpose:  This function sets NUMBER OF 8A-GROUPS.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_SetAudioBCTMCNumberOf8AGroups (ViSession instrSession,
                                                       ViInt32 numberOf8AGroups)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_TMC_NUMBER_OF_8A_GROUPS,
                            numberOf8AGroups), 2, "Number Of 8A Groups");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure Audio BC FM TMC 8A Block
/* Purpose:  This function configures TMC group 8A.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCFMTMC8ABlock (ViSession instrSession,
                                                      ViInt32 group8A,
                                                      ViInt32 block,
                                                      ViInt32 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";

    if ((group8A < 1) || (group8A > 6))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group 8A"); 
    if ((block < 2) || (block > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "G8A%ld,B%ld", group8A, block);    
        
    if (block == 2) {
            if ((value < 0) || (value > 31))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value"); 
    }
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_AUDIO_BC_TMC_8A_BLOCK,
                    value),  4, "Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Enable Audio BC FM Open Format
/* Purpose:  This function switches the open format on or off.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_EnableAudioBCFMOpenFormat (ViSession instrSession,
                                                   ViBoolean openFormat)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_OPEN_FORMAT, openFormat),
                         2, "Open Format");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;           
}

/*===========================================================================*/
/* Function: Audio BC FM Open Format Send Data
/* Purpose:  This function sends the open format data (FM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_AudioBCFMOpenFormatSendData (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));    

    viCheckErr (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_AUDIO_BC_OPEN_FORMAT_APPLY, "")); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 
/*===========================================================================*/
/* Function: Configure Audio BC Open Format Group A
/* Purpose:  This function configures open format Group A.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCOpenFormatGroupA (ViSession instrSession,
                                                          ViInt32 groupA,
                                                          ViInt32 block,
                                                          ViInt32 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";
              
    // only group: 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15
    if ((groupA < 1) || (groupA > 15))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group A"); 
    if ((block < 2) || (block > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "G%ldA,B%ld", groupA, block); 
        
    if (block == 2) {
            if ((value < 0) || (value > 31))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value"); 
    }
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A,
                    value),  4, "Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Configure Audio BC Open Format Group B
/* Purpose:  This function configures open format Group B.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ConfigureAudioBCOpenFormatGroupB (ViSession instrSession,
                                                          ViInt32 groupB,
                                                          ViInt32 block,
                                                          ViInt32 value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";
              
    // only group: 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13
    if ((groupB < 1) || (groupB > 13))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group B"); 
    if ((block < 2) || (block == 3) || (block > 4))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Block"); 
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    sprintf (buffer, "G%ldB,B%ld", groupB, block);    
        
    if (block == 2) {
            if ((value < 0) || (value > 31))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Value"); 
    }
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B,
                    value),  4, "Value"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
} 

/*===========================================================================*/
/* Function: Audio BC Open Format Group Data Transfer Status
/* Purpose:  This function reads the data transfer status of the indicated
/*           open format group.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_AudioBCOpenFormatGroupDataTransferStatus
             (ViSession instrSession, ViInt32 group, ViInt32 groupType,
              ViInt32* transferStatus)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";


    checkErr( Rs_LockSession (instrSession, VI_NULL));
   
    switch (groupType){
        case RSSFC_VAL_AUDIO_BC_OF_GROUP_A: {
            if ((group < 1) || (group > 15))    // only group: 1, 3, 5, 6, 7, 8, 9, 11, 12, 13, 15 
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group"); 
            sprintf (buffer, "G%ldA", group);     
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_A_READY, 
                        transferStatus), 4, "Transfer Status");
        }
        break;
        
        case RSSFC_VAL_AUDIO_BC_OF_GROUP_B: {
            if ((group < 1) || (group > 13))    // only group: 1, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13
                viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Group"); 
            sprintf (buffer, "G%ldB", group); 
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, buffer, RSSFC_ATTR_AUDIO_BC_OPEN_FORMAT_GROUP_B_READY, 
                        transferStatus), 4, "Transfer Status");
        }
        break;

        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Group Type");
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Configure Audio BC FM Audio Modulation
/* Purpose:  This function configures Audio Broadcasting modulation 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configAudioBCFMAudioModulation(
    ViSession   instrSession,
    ViInt32 mode,
    ViInt32 preemphasis,
    ViReal64    nomFreqDevAudio
)
{
 ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_MODE, mode),
                         2, "Mode"); 
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_PREEMPHASIS, preemphasis),
                         3, "Preemphasis");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_NOM_FREQ_DEV_AUDIO, nomFreqDevAudio),
                         4, "Nom Freq Dev Audio");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;       
}


/*===========================================================================*/
/* Function: Configure Audio BC FM Modulation RDS
/* Purpose:  This function configures audio modulation coding parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configureAudioBCModulationRDS(
    ViSession   instrSession,
    ViBoolean   rds,
    ViReal64    freqDevRDS,
    ViReal64    freqDevPilot19kHz
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_RDS, rds),
                         2, "RDS");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_RDS, freqDevRDS),
                         3, "Freq Dev RDS");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_PILOT_19KHZ, freqDevPilot19kHz),
                         4, "Freq Dev Pilot 19kHz");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;           
}


/*===========================================================================*/
/* Function: Read Audio BC FM Modulation Values
/* Purpose:  This function reads selected modulation value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readAudioBCFMModulationValues(
    ViSession   instrSession,
    ViInt32 value,
    ViReal64*   result
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    switch (value){
        case RSSFC_VAL_AUDIO_FREQ_DEV_AUDIO:
            viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_MODULATION_FREQ_DEV_AUDIO, 
                result), 3, "Result");
        break;
        
        default:
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Value");
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Enable Audio BC FM Special Settings
/* Purpose:  This function enables use of special Audio BC FM coding 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_enableAudioBCFMSpecialSettings(
    ViSession   instrSession,
    ViBoolean   state
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_FM_SPECIAL_SETTINGS, state),
                         2, "State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Configure  Audio BC FM Special Settings
/* Purpose:  This function configures special Audio BC coding parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confAudioBCFMSpecialSettings(
    ViSession   instrSession,
    ViBoolean   pilot,
    ViReal64    phaseOffsetPilot19kHz,
    ViReal64    phaseOffsetRDS
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_AUDIO_BC_SPECIAL_PILOT, pilot),
                         2, "Pilot");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SPECIAL_PHASE_OFFSET_PILOT_19KHZ, phaseOffsetPilot19kHz),
                         3, "Phase Offset Pilot 19kHz");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_SPECIAL_PHASE_OFFSET_RDS, phaseOffsetRDS),
                         4, "Phase Offset RDS");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC AM Input Signal
/* Purpose:  This function configures Audio Broadcasting AM Input Signal 
/*           parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_configAudioBCAMInputSignal(
    ViSession   instrSession,
    ViInt32 source
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AM_SOURCE, source),
                         2, "Source"); 
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: Configure Audio BC AM Source
/* Purpose:  This function configures Audio BC AM parameters.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confAudioBCAMSource(
    ViSession   instrSession,
    ViReal64    frequency,
    ViReal64    level
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AM_INTERNAL_FREQUENCY, frequency),
                         2, "Frequency");
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AM_INTERNAL_LEVEL, level),
                         3, "Level");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}


/*===========================================================================*/
/* Function: Configure Audio BC AM Modulation Depth
/* Purpose:  This function configures the nominal modulation depth (AM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_confAudioBCAMModulationDepth(
    ViSession   instrSession,
    ViReal64    nomModDepth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AM_NOM_MOD_DEPTH, nomModDepth),
                         2, "Nom Mod Depth");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read Audio BC AM Mode Depth
/* Purpose:  This function queries the nominal modulation depth (AM).
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readAudioBCAMModeDepth(
    ViSession   instrSession,
    ViReal64*   modeDepth
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_AUDIO_BC_AM_MOD_DEPTH, 
               modeDepth), 2, "Mode Depth");

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
     



/*****************************************************************************
 *---------------------------- Noise ----------------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Noise Configuration 
 * Purpose:  This function configures the noise generator.
/============================================================================*/
ViStatus _VI_FUNC rssfc_NoiseConfiguration (ViSession instrSession, ViInt32 noise,
                                        ViReal64 cN, ViBoolean bandwidthCoupling,
                                        ViReal64 receiverBandwidth)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_NOISE, noise),
                         2, "Noise"); 
	
	if (noise != RSSFC_VAL_NOISE_OFF ) {
	
		if (noise != RSSFC_VAL_NOISE_ONLY) {
			viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_NOISE_CN, cN), 3, "C/N");
		}
		
		viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_NOISE_BANDWIDTH_COUPLING, bandwidthCoupling),
	                         4, "Bandwidth Coupling");
	
	    if (bandwidthCoupling == VI_FALSE)
	    {
	        viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_NOISE_RECEIVER_BANDWIDTH, receiverBandwidth),
	                             5, "Receiver Bandwidth");                         
	    }
	}
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Gen Noise State
 * Purpose:  This function queries the noise generator state.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readNoiseGenState (ViSession instrSession,
                                           ViInt32* noiseState)
{
    //ViChar noiseState[])  
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!noiseState) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Noise State");
    
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_NOISE, 
                        noiseState), 2, "Noise State");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Configure Noise CN
 * Purpose:  This function sets the carrier/noise ratio (C/N).
/============================================================================*/
ViStatus _VI_FUNC rssfc_confNoiseCn (ViSession instrSession, ViReal64 cN)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_NOISE_CN, cN), 2, "C/N");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Noise CN
 * Purpose:  This function queries the carrier/noise ratio.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readNoiseCN (ViSession instrSession, ViReal64 *cnr)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!cnr) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "CNR");
    viCheckErr (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_NOISE_CN,
                                            cnr));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read Gen Noise Bandwidth
 * Purpose:  This function queries the generated noise bandwidth.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readNoiseGenBand (ViSession instrSession,
                                          ViReal64 *generatedNoiseBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (!generatedNoiseBandwidth) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Generated Noise Bandwidth");
    viCheckErr (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_GEN_NOISE_BANDWIDTH,
                                            generatedNoiseBandwidth));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *------------------------- TSGEN ---------------------------------*
 *****************************************************************************/
 
/*============================================================================/
 * Function: TSPlayer and Recorder Status
 * Purpose:  Select the status of the TSPlayer/Recorder.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actTsgenStatus (ViSession instrSession,
                                        ViInt32 command)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TSGEN_COMMAND, 
                command), 2, "Command");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Read TSPlayer and Recorder Status
 * Purpose:  Queries the actual state of the TSPlayer or Recorder.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readTsgenStatus (ViSession instrSession,
                                         ViInt32* command)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TSGEN_COMMAND_STATE, 
                command), 2, "Command");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: Read Player Parameters
 * Purpose:  This function reads player parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_readTsgenPlayerParam (ViSession instrSession,
                                         ViChar  fileDateSize[],
                                         ViChar  originalLoopTime[],
                                         ViReal64*  originalTSDataRate)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE]="\0";
 
    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if (fileDateSize == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for File Date / Size");
    if (originalLoopTime == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Original Loop Time");
    if (originalTSDataRate == VI_NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Original TS Data Rate");
        

    viCheckParm (rssfc_GetAttributeViString (instrSession, "", RSSFC_ATTR_TSGEN_FILE_DATE, 
                        RSSFC_IO_BUFFER_SIZE, buffer), 2, "File Date / Size");
    strncpy (fileDateSize, buffer, strlen(buffer));
    
    
    viCheckParm (rssfc_GetAttributeViString (instrSession, "", RSSFC_ATTR_TSGEN_ORIGINAL_LOOP_TIME, 
                        RSSFC_IO_BUFFER_SIZE, buffer), 3, "Original Loop Time");
    strncpy (originalLoopTime, buffer, strlen(buffer));  
    
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_ORIGINAL_TS_DATA_RATE, 
                        originalTSDataRate), 4, "Original TS Data Rate");
                
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 * Function: File Operations
 * Purpose:  The MMEMory subsystem (Mass Memory) contains the commands for
 *           managing files and directories as well as for loading and
 *           storing complete instrument settings in files.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_actFileOperations (ViSession instrSession,
                                           ViInt32 operation,
                                           ViString source,
                                           ViString destination)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    switch (operation){
        case RSSFC_VAL_MMEM_COPY:
            viCheckErr( viPrintf (instrSession, "MMEM:COPY '%s','%s'", source, destination)); 
        break;
        
        case RSSFC_VAL_MMEM_DELETE:
            viCheckErr( viPrintf (instrSession, "MMEM:DEL '%s'", source)); 
        break;
        
        case RSSFC_VAL_MMEM_RENAME:
            viCheckErr( viPrintf (instrSession, "MMEM:MOVE '%s','%s'", source, destination)); 
        break;
        
        case RSSFC_VAL_MMEM_MDIR:
            viCheckErr( viPrintf (instrSession, "MMEM:MDIR '%s'", source)); 
        break;
        
        case RSSFC_VAL_MMEM_CDIR:
            viCheckErr( viPrintf (instrSession, "MMEM:CDIR '%s'", source)); 
        break;        
        
        case RSSFC_VAL_MMEM_RDIR:
            viCheckErr( viPrintf (instrSession, "MMEM:RDIR '%s'", source)); 
        break;
        
        case RSSFC_VAL_MMEM_MSIS:
            viCheckErr( viPrintf (instrSession, "MMEM:MSIS '%s'", source)); 
        break;        
        }
    viCheckErr(rssfc_CheckStatus (instrSession));
    }
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================
 * Function: File Directory Path                                             
 * Purpose:  This fuction reads out the subdirectories or subdirectories and
 *           files in the specified directory.                        
/============================================================================*/
ViStatus _VI_FUNC rssfc_actFileCatPath (ViSession instrSession, ViInt32 read,
                                        ViString directory, ViInt32 bufferSize,
                                        ViChar output[], ViInt32 *returnedValues)
{
   
    ViChar*     pbuffer = NULL;
    ViUInt32    count = 0;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (bufferSize != 0 && output == VI_NULL)
        return VI_ERROR_PARAMETER5;

    if (read == RSSFC_VAL_MMEM_CATALOG_FILES)
    {
        viCheckErr( viPrintf(instrSession, "*CLS;:MMEM:CAT? '%s'\n", directory));
    }
    else
        viCheckErr( viPrintf(instrSession, "*CLS;:MMEM:DCAT? '%s'\n", directory)); 
    
    
    viCheckErr( rssfc_WaitForOPC (instrSession, MAX_OPC_TIME)); 
    viCheckErr( rssfc_ReadDataUnknownLength(instrSession, &pbuffer, &count));
    
    if (output != NULL) strncpy(output, pbuffer, (size_t) ((bufferSize > (ViInt32) count) ? (ViInt32) count : bufferSize));
    free(pbuffer);
    if (returnedValues) *returnedValues = (ViInt32) count;
        
    viCheckErr(rssfc_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*===========================================================================*/
/* Function: TSGen Seek Position
/* Purpose:  Queries the measured data rate of the transport stream at the 
/*           selected recorder input.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_readTSGenSeekPosition(
    ViSession   instrSession,
    ViReal64*   position
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_SEEK_POSITION,
                                            position), 2, "Position");    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;   
}

/*============================================================================/
 * Function: TSGen Nullpacket Stuffing
 * Purpose:  This function sets the output data rate of the TS Player.
/============================================================================*/
ViStatus _VI_FUNC rssfc_TSGenNullpacketStuffing (ViSession instrSession,
                                         ViBoolean NullpacketStuffing)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
                                                     
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TSGEN_PLAYER_NULLPACKET_STUFFING , 
                NullpacketStuffing), 2, "Nullpacket Stuffing");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: TSGen Packed Length
 * Purpose:  This function determines the packet length with which the TS
 *           data stream is replayed.
/============================================================================*/
ViStatus _VI_FUNC rssfc_TSGenPackedLength(ViSession instrSession,
                                         ViInt32* packedLength)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if (!packedLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Packed Length");   
        
    viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_TSGEN_PACKET_LENGTH, 
                packedLength), 2, "Packed Length");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Catalog Length
 * Purpose:  This function reads out the number of files or subdirectories in
 *           the specified directory.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readCatalogLength (ViSession instrSession, ViInt32 read,
                                         ViInt32 *catalogLength)
{
   ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    if (!catalogLength) viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Catalog Length");
    
    if (read == RSSFC_VAL_MMEM_CATALOG_FILES)
    {
        viCheckErr( viQueryf (instrSession, "MMEM:CAT:LENG?", "%ld", catalogLength));
    }
    else
        viCheckErr( viQueryf (instrSession, "MMEM:DCAT:LENG?", "%ld", catalogLength));
 }   
    viCheckErr(rssfc_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


///////////////////////////////////////////////////////////////////////////////////

/*============================================================================/
 * Function: Read To File From Instrument
 * Purpose:  This function is used to read data from the instrument and
 *           write it to a user specified file.
/============================================================================*/
ViStatus _VI_FUNC rssfc_readToFile (ViSession instrSession, ViString source,
                ViString destination)
{
    ViUInt32    retCnt = 0;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";
    ViChar      *exBuf;
    ViInt32     lenLength = 0;
    size_t      count = 0;
    ViInt32     cnt = 0;
    FILE        *targetFile;
    //exBuf = 0;

    ViStatus error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */
    //
    sprintf (buffer, "*CLS;:MMEM:DATA? '%s'\n", source);


    viCheckErr( viPrintf (instrSession, "%s", buffer));    
    viCheckErr( rssfc_WaitForOPC (instrSession, MAX_OPC_TIME)); 

    /* GPIB Interface specific operations */
   

    viCheckErr( viRead (instrSession, (ViBuf) buffer, 2, &retCnt));
    
    if (sscanf (buffer, "#%1ld", &lenLength) != 1) {
        error = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return error;
        }

    viCheckErr( viRead (instrSession, (ViBuf) buffer, (ViUInt32) lenLength, &retCnt));

        buffer [lenLength]='\0';

    if (sscanf (buffer, "%ld", &cnt) != 1) {
        error = VI_ERROR_INSTR_INTERPRETING_RESPONSE;
        return error;
        }

    count = (size_t) cnt;
    /* Allocate memory space */
    exBuf = malloc (count + 1);
    if (exBuf == NULL)
       return VI_ERROR_ALLOC;
    else
        exBuf [0] = '\0';
    
    if ((error = viRead (instrSession, (ViBuf) exBuf, (ViUInt32) (count + 1), &retCnt)) < 0)
    {
    /* If not NULL pointer, then deallocate memory. */
        if (exBuf != NULL) free (exBuf);
           return error;
        }

    /* Store data from instrument to file on host computer */
    targetFile = fopen (destination, "wb");
    if (targetFile == NULL)
        {
        if (exBuf != NULL)
            free (exBuf);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    if (count != fwrite (exBuf, sizeof (ViByte), count, targetFile))
        {
        (void) fclose (targetFile);
        if (exBuf != NULL)
            free (exBuf);
        return VI_ERROR_INSTR_FILE_WRITE;
        }

    (void) fclose (targetFile);

    if (exBuf != NULL) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    viCheckErr(rssfc_CheckStatus (instrSession)); 
    }
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;  
}

/*============================================================================/
 * Function: Write From File To Instrument
 * Purpose:  This function is used to read data from a user specified file
 *           and write it to the instrument.
/============================================================================*/
ViStatus _VI_FUNC rssfc_writeFromFile (ViSession instrSession, ViString source,
                ViString destination)
{
    ViInt32     digits;

    ViUInt32    fileSize            = 0;
    ViUInt32    bytesWrite          = 0;
    ViChar      *exBuf;
    ViChar      *p2buf;
    FILE        *sourceFile;

//

   ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0){                /* call only when locked */

    /* Open source file from host controller */
    sourceFile = fopen (source, "rb");
    if (sourceFile == NULL)
        return VI_ERROR_INSTR_FILE_OPEN;

    /* Get size of file */
    (void)fseek (sourceFile, 0, SEEK_END);
    fileSize = (ViUInt32) ftell (sourceFile);
    (void)fseek (sourceFile, 0, SEEK_SET);

    /* Allocate memory space (add 64 bytes for command header) */
    exBuf = malloc ((size_t)(fileSize + 64));
    if (exBuf == NULL)
        return VI_ERROR_ALLOC;
    else
        exBuf[0] = '\0';

    /* Create command header */
    digits = ((int)floor(log10((double)fileSize))) + 1;
    p2buf = exBuf;
    bytesWrite = (ViUInt32) sprintf (p2buf, ":MMEM:DATA '%s',#%ld%lu", destination, digits, fileSize);
    p2buf += bytesWrite;

    if (fileSize != (ViUInt32) fread (p2buf, sizeof (ViByte), (size_t)fileSize, sourceFile))
        {
        free (exBuf);
        return VI_ERROR_INSTR_FILE_OPEN;
        }

    bytesWrite += fileSize;
    (void)fclose (sourceFile);

  
             if ((error = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:LTER EOI\n", 25, VI_NULL)) < 0)
                 
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return error;
                }
            if ((error = viWrite (instrSession, (ViBuf) exBuf, bytesWrite, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return error;
                }
            if ((error = viWrite (instrSession,
                (ViBuf) ":SYST:COMM:GPIB:LTER LSTAN\n", 27, VI_NULL)) < 0)
                {
                /* If not NULL pointer, then deallocate memory. */
                free (exBuf);
                return error;
                }



    if (exBuf) /* If not NULL pointer, then deallocate memory. */
        free (exBuf);

    viCheckErr(rssfc_CheckStatus (instrSession));
    }
    

Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Select Play File
 * Purpose:  This function selects the file to be loaded into the TS Player
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTsgenPlayfile (ViSession instrSession,
                                           ViString fileName)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    viCheckParm (rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_TSGEN_PLAYER_PLAYFILE, fileName),
                         2, "File name");                           
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*===========================================================================*/
/* Function: Set TS Byte Rate 
/* Purpose:  This function sets the TS byte rate.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_setTsByteRate(
    ViSession   instrSession,
    ViReal64    byteRate
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_PLAYER_BYTERATE, byteRate),
                         2, "Byte Rate");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Timing Settings
 * Purpose:  This function configures timing settings.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confTsgenTimingSettings (ViSession instrSession,
                                            ViReal64 TSDataRate,
                                            ViReal64 playWindowStart,
                                            ViReal64 playWindowStop)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_TS_DATA_RATE, TSDataRate),
                         2, "TS Data Rate"); 
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_PLAY_WINDOW_START, playWindowStart),
                         3, "Play Window Start"); 
    viCheckParm (rssfc_SetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_PLAY_WINDOW_STOP, playWindowStop),
                         4, "Play Window Stop");    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*===========================================================================*/
/* Function: Read TS Data Rate 
/* Purpose:  This function reads selected TS Data Rate value.
/*===========================================================================*/
ViStatus _VI_FUNC rssfc_ReadTSDataRate(
    ViSession   instrSession,
    ViReal64*    tsDataRate
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    viCheckParm (rssfc_GetAttributeViReal64 (instrSession, "", RSSFC_ATTR_TSGEN_TS_DATA_RATE, tsDataRate),
                         2, "TS DataRate");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}	

/*============================================================================/
 * Function: TSGen Configure Seamless Loop
 * Purpose:  This function configures seamless loop parameters.
/============================================================================*/
ViStatus _VI_FUNC rssfc_TSGenConfigureSeamlessLoop (ViSession instrSession,
                                                    ViBoolean continuityCounter,
                                                    ViBoolean PCRDTSPTS,
                                                    ViBoolean TDTTOT)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TSGEN_SEAMLESS_LOOP_CONT_COUNT, 
                continuityCounter), 2, "continuityCounter");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TSGEN_SEAMLESS_LOOP_PCR_DTS_PTS, 
                PCRDTSPTS), 3, "PCR,DTS/PTS");
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_TSGEN_SEAMLESS_LOOP_TDT_TOT, 
                TDTTOT), 4, "TDT/TOT");
                    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*****************************************************************************
 *------------------------- Adjustment --------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: Configure Internal Level Correction
 * Purpose:  This function switches internal level correction on or off.
/============================================================================*/
ViStatus _VI_FUNC rssfc_confCalInternal(ViSession instrSession,
                                        ViBoolean internalLevelCorrection)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_CAL_LEV_STATE, 
                internalLevelCorrection), 2, "Internal Level Correction");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Run Calibration
 * Purpose:  This function runs selected calibration.
/============================================================================*/
ViStatus _VI_FUNC rssfc_actCal(ViSession   instrSession,
                               ViInt32 calibrationType,
                               ViInt32*    result)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     opcTimeout;
    ViInt32     timeout = 210000;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((calibrationType < RSSFC_VAL_CAL_ALL) || (calibrationType > RSSFC_VAL_CAL_VMF))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Calibration Type");
    
    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    checkErr (Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0, &timeout));
    switch (calibrationType){
        case RSSFC_VAL_CAL_ALL:
            viCheckErr( rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_CAL_ALL, ""));
                viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CAL_ALL_RESULT, 
                        result), 3, "Result");
        break;
        
        case RSSFC_VAL_CAL_VML:
            viCheckErr( rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_CAL_LOCAL, ""));
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CAL_LOCAL_RESULT, 
                        result), 3, "Result");
        break;
       
        case RSSFC_VAL_CAL_VMF:
            viCheckErr( rssfc_SetAttributeViString (instrSession, "", RSSFC_ATTR_CAL_IQM_FULL, ""));
            viCheckParm (rssfc_GetAttributeViInt32 (instrSession, "", RSSFC_ATTR_CAL_IQM_FULL_RESULT, 
                        result), 3, "Result");
        break;
    }

Error:
    Rs_SetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0, &opcTimeout);
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: Calibration Date
 * Purpose:  This function queries the date of the last system error 
 *           correction.
/============================================================================*/
ViStatus _VI_FUNC rssfc_getCalDate(ViSession   instrSession,
                                   ViInt32 calibrationType,
                                   ViChar  date[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = "";
    
    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    if ((calibrationType < RSSFC_VAL_CAL_VML) || (calibrationType > RSSFC_VAL_CAL_VMF))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Calibration Type");
        
    switch (calibrationType){
        case RSSFC_VAL_CAL_VML:
            viCheckErr (rssfc_GetAttributeViString (instrSession, "", RSSFC_ATTR_CAL_LOCAL_DATE, RSSFC_IO_BUFFER_SIZE,
                        buffer));
        break;
        case RSSFC_VAL_CAL_VMF:
            viCheckErr (rssfc_GetAttributeViString (instrSession, "", RSSFC_ATTR_CAL_IQM_FULL_DATE, RSSFC_IO_BUFFER_SIZE,
                        buffer));
        break;
        default: buffer[0] = '\0';
    }   
    strncpy (date, buffer, strlen(buffer));
    
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *------------------------- System  -----------------------------------------*
 *****************************************************************************/
/*============================================================================/
 * Function: System Display Update
 * Purpose:  This function activates/deactivates updating of the display.
 *           Upon changing to manual control, the status is automatically set
 *           to ON.
/============================================================================*/
ViStatus _VI_FUNC rssfc_systemDisplayUpdate (ViSession instrSession,
                                             ViBoolean displayUpdate)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_SYSTEM_DISPLAY_UPDATE, 
                displayUpdate), 2, "Display Update");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}

/*============================================================================/
 * Function: System Keypad Lock
 * Purpose:  This function locks or unlocks the front panel keypad of the
 *           R&S SFE including the LOCAL key.
/============================================================================*/
ViStatus _VI_FUNC rssfc_systemKeypadLock (ViSession instrSession,
                                          ViInt32 keypadLock)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_SYSTEM_KLOCK, 
                keypadLock), 2, "Keypad Lock");
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*============================================================================/
 * Function: Set Status Register
 * Purpose:  This function sets the Enable, NTransition, and PTransition bits
 *           of status questionable and operating registers.
/============================================================================*/
ViStatus _VI_FUNC rssfc_setStatusRegister (ViSession instrSession,
                                           ViInt32 registerOperation,
                                           ViInt32 enable, ViInt32 PTransition,
                                           ViInt32 NTransition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSSFC_VAL_PRESET_STAT_REG) || (registerOperation > RSSFC_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
        
    if ((enable < 0) || (enable > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Enable");
    
    if ((PTransition < 0) || (PTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "PTransition");     
    
    if ((NTransition < 0) || (NTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "NTransition");     
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSSFC_VAL_PRESET_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:PRES")); 
            break;
            
            case RSSFC_VAL_QUEST_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:QUES:ENAB %ld;PTR %ld;NTR %ld",
                    enable, PTransition, NTransition)); 
            break;
            
            case RSSFC_VAL_OPER_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:OPER:ENAB %ld;PTR %ld;NTR %ld",
                    enable, PTransition, NTransition)); 
            break;
        }
    }   
    
    viCheckErr(rssfc_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Get Status Register
 * Purpose:  This function contains the commands for the querying status
 *           reporting system.
/============================================================================*/
ViStatus _VI_FUNC rssfc_getStatusRegister (ViSession instrSession,
                                           ViInt32 registerOperation,
                                           ViInt32 statusRegistersQuery,
                                           ViInt32* registerValue)

{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSSFC_VAL_QUEST_STAT_REG   ) || (registerOperation > RSSFC_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if ((statusRegistersQuery < RSSFC_VAL_REG_CONDITION   ) || (statusRegistersQuery > RSSFC_VAL_REG_NTRANSITION))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Register Operation");  
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSSFC_VAL_QUEST_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:QUES:%s?", "%ld",
                    statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
            
            case RSSFC_VAL_OPER_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:OPER:%s?", "%ld",
                    statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
        }
    }   

    viCheckErr(rssfc_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
                                           

/*============================================================================/
 * Function: Set User-definable Status Register
 * Purpose:  This function sets the Enable, NTransition and PTransition
 *           user-definable bits of status questionable and operating registers.
/============================================================================*/
ViStatus _VI_FUNC rssfc_setUserDefStatusRegister (ViSession instrSession,
                                                  ViInt32 registerOperation,
                                                  ViInt32 userdefinableBit,
                                                  ViInt32 enable,
                                                  ViInt32 PTransition,
                                                  ViInt32 NTransition)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSSFC_VAL_QUEST_STAT_REG) || (registerOperation > RSSFC_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");

    if (registerOperation == RSSFC_VAL_QUEST_STAT_REG) {
        if ((userdefinableBit < 9) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    else {
        if ((userdefinableBit < 8) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    
    if ((enable < 0) || (enable > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Enable");
    
    if ((PTransition < 0) || (PTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 5, "PTransition");     
    
    if ((NTransition < 0) || (NTransition > 32767))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 6, "NTransition");     
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {

            case RSSFC_VAL_QUEST_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:QUES:BIT%ld:ENAB %ld;PTR %ld;NTR %ld",
                    userdefinableBit, enable, PTransition, NTransition)); 
            break;
            
            case RSSFC_VAL_OPER_STAT_REG:
                viCheckErr(viPrintf (instrSession, ":STAT:OPER:BIT%ld:ENAB %ld;PTR %ld;NTR %ld",
                    userdefinableBit, enable, PTransition, NTransition)); 
            break;
        }
    }   
    
    viCheckErr(rssfc_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
 

/*============================================================================/
 * Function: Get User-definable Status Register
 * Purpose:  This function contains the commands for the querying status
 *           reporting system.
/============================================================================*/
ViStatus _VI_FUNC rssfc_getUserDefStatusRegister (ViSession instrSession,
                                                  ViInt32 registerOperation,
                                                  ViInt32 userdefinableBit,
                                                  ViInt32 statusRegistersQuery,
                                                  ViInt32* registerValue)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((registerOperation < RSSFC_VAL_QUEST_STAT_REG   ) || (registerOperation > RSSFC_VAL_OPER_STAT_REG))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Register Operation");
    
    if (registerOperation == RSSFC_VAL_QUEST_STAT_REG) {
        if ((userdefinableBit < 9) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    else {
        if ((userdefinableBit < 8) || (userdefinableBit > 12))
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "User-definable Bit");
    }
    
    if ((statusRegistersQuery < RSSFC_VAL_REG_CONDITION   ) || (statusRegistersQuery > RSSFC_VAL_REG_NTRANSITION))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Register Operation");  
            
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    
        switch (registerOperation) {
            case RSSFC_VAL_QUEST_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:QUES:BIT%ld:%s?", "%ld",
                    userdefinableBit, statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
            
            case RSSFC_VAL_OPER_STAT_REG:
                viCheckErr (viQueryf (instrSession, ":STAT:OPER:BIT%ld:%s?", "%ld",
                    userdefinableBit, statusRegisterArr[statusRegistersQuery], registerValue)); 
            break;
        }
    }   

    viCheckErr(rssfc_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}                                          
                                           
    
                                           
/*****************************************************************************
 *--------------------- Hardware Settings -----------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Configure Reference Oscillator
 * Purpose:  This function configures reference oscillator.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_confReferenceOscillator (ViSession instrSession,
                                                 ViInt32 source,
                                                 ViBoolean adjustmentState,
                                                 ViInt32 adjustmentFrequency)
{
    ViStatus    error = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
    
    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ROSC_SOURCE, source),
                         2, "Source");
	if (source == RSSFC_VAL_ROSC_SOURCE_INT) {
	    viCheckParm (rssfc_SetAttributeViBoolean (instrSession, "", RSSFC_ATTR_ROSC_ADJ_STATE, adjustmentState),
	                         3, "Adjustment State");                          
	    viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_ROSC_ADJ_FREQUENCY, adjustmentFrequency),
	                         4, "Adjustment Frequency"); 
	}
                         
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 *---------------------------------------------------------------------------*
 *****************************************************************************/

/*============================================================================/
 * Function: Store and Load Application
 * Purpose:  This function stores or recalls application settings in the
 *           specified file.
/============================================================================*/
ViStatus _VI_FUNC rssfc_storeLoadApplication (ViSession instrSession,
                                                         ViInt32 operation,
                                                         ViString source)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSSFC_IO_BUFFER_SIZE] = ""; 

    checkErr( Rs_LockSession (instrSession, VI_NULL));

    if ((operation < RSSFC_VAL_SAVE) || (operation > RSSFC_VAL_RECALL))
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Operation");
        
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */

        switch (operation){
            case RSSFC_VAL_SAVE:
                 sprintf (buffer,"*SAV 1;:MMEMory:STORe:STATe 1,\"%s\"", source);
            break;
        
            case RSSFC_VAL_RECALL:
                sprintf (buffer,":MMEMory:LOAD:STATe 1,\"%s\";*RCL 1", source);
            break;
        }   
    }
    viCheckErr( viPrintf (instrSession, "%s", buffer));         
    viCheckErr(rssfc_CheckStatus (instrSession));       
     
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}
     

/*============================================================================/
 * Function: Store and Load Instrument Data
 * Purpose:  This function stores or recalls instrument data.
/============================================================================*/
ViStatus _VI_FUNC rssfc_storeLoadInstrumentData (ViSession instrSession,
                                                 ViInt32 networkMode,
                                                 ViInt32 stateNumber)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, VI_NULL));
  
    if (Rs_Simulating(instrSession) == 0) {                /* call only when locked */
    switch (networkMode){
        case RSSFC_VAL_SAVE:
             viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_SAV, stateNumber),
                         3, "State Number"); 
        break;
        
        case RSSFC_VAL_RECALL:
             viCheckParm (rssfc_SetAttributeViInt32 (instrSession, "", RSSFC_ATTR_RCL, stateNumber),
                         3, "State Number"); 
        break;
        }
    }
Error:
    (void) Rs_UnlockSession(instrSession, VI_NULL);
    return error;
}


/*****************************************************************************
 * Function: rssfc_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfc_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfc_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rssfc_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    memset (value, 0, bufSize);
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfc_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rssfc_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}

/*****************************************************************************
 * Function: rssfc_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfc_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfc_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rssfc_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfc_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rssfc_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rssfc_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the RSSFC_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rssfc_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rssfc_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rssfc_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViBoolean */
ViStatus _VI_FUNC rssfc_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rssfc_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                /*@unused@*/ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViUInt32     rcap;
	attributeID;

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    for (rcap = 0; rcap < sessionProperties -> numberOfRepCaps; rcap++)
        if (strncmp (sessionProperties -> repCapTable[rcap].repCapNameId, repeatedCapabilityId, strlen (repeatedCapabilityId)) == 0)
            break;

    strncpy (repeatedCapabilityIdNames, sessionProperties -> repCapTable[rcap].repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rssfc_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rssfc_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, (ViUInt32 *) bytesRead);
}

/*****************************************************************************
 *-------------------- Utility Functions (Not Exported) ---------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rssfc_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           RSSFC_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus rssfc_CheckStatus (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    if ((Rs_QueryInstrStatus (instrSession) != 0) && (Rs_Simulating (instrSession) == 0))
        {
        checkErr( rssfc_CheckStatusCallback (instrSession));
        }

Error:
    return error;
}

/*****************************************************************************
 * Function: rssfc_WaitForOPC
 * Purpose:  This function waits for the instrument to complete the
 *           execution of all pending operations. This function is a
 *           wrapper for the WaitForOPCCallback. It can be called from
 *           other instrument driver functions.
 *
 *           The maxTime parameter specifies the maximum time to wait for
 *           operation complete in milliseconds.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus rssfc_WaitForOPC (ViSession instrSession, ViInt32 maxTime)
{
    ViStatus    error = VI_SUCCESS;

    if (Rs_Simulating(instrSession) == 0)
        {
        ViInt32  oldOPCTimeout = 0;

        checkErr( Rs_GetAttribute (instrSession, VI_NULL, RSSFC_ATTR_OPC_TIMEOUT, 0,
                                   (ViInt32) sizeof (oldOPCTimeout), &oldOPCTimeout));

        (void)Rs_SetAttribute (instrSession, VI_NULL, RSSFC_ATTR_OPC_TIMEOUT, 0, &maxTime);

        error = rssfc_WaitForOPCCallback (instrSession);

        (void)Rs_SetAttribute (instrSession, VI_NULL, RSSFC_ATTR_OPC_TIMEOUT, 0, &oldOPCTimeout);

        viCheckErr( error);
        }
Error:
    return error;
}

/*****************************************************************************
 * Function: rssfc_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rssfc_reset function calls this function. The
 *           rssfc_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
static ViStatus rssfc_DefaultInstrSetup (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

    /* Set all the attributes to the default state. Do not update inherent attributes! */
    checkErr (Rs_ApplyAttributeDefaults (instrSession, VI_FALSE));

    /* Init attributes */
    checkErr( rssfc_InitAttributes (instrSession));

    if (Rs_Simulating(instrSession) == 0)
        {
        viCheckErr( viPrintf (instrSession, "*CLS;*ESE 1;*SRE 32"));
        }

Error:
    return error;
}

/******************************************************************************
 * Function: Read Data With Unknown length
 * Description: This function allocate buffer and read data from the instrument
 * WARNING:  You must deallocate buffer, if any pointer has been returned.
 *           This function does not work correctly, when you read bin data via
 *           serial interface
 *****************************************************************************/
static ViStatus rssfc_ReadDataUnknownLength(ViSession instrSession,
                ViChar **outBuffer, ViUInt32 *count)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = VI_NULL, *p2buffer;

    // *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = (ViUInt32) RSSFC_IO_BUFFER_SIZE;
    buffer = (ViChar *)calloc((size_t)actSize, sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
        {
        if ((error = viRead (instrSession, (ViBuf) p2buffer, (ViUInt32) RSSFC_IO_BUFFER_SIZE, &retCnt)) < 0)
            {
            free(buffer);
            return error;
            }
        if (retCnt == (ViUInt32) RSSFC_IO_BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += RSSFC_IO_BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - RSSFC_IO_BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string on the end of data */
        }
       } while (error == 0x3FFF0006); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count) *count = (actSize - RSSFC_IO_BUFFER_SIZE) + retCnt;

    return error;
}


/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           rssfc_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rssfc_CheckStatusCallback (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer [10] = "*ESR?\n";
    ViInt16     esr     = 0;

    viCheckErr (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen (buffer), NULL));
    viCheckErr (viRead (instrSession, (ViBuf)buffer, 10, NULL));
    
    sscanf (buffer, "%hd", &esr);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
        {
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
        }

Error:

    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rssfc_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;

#if 0 // --- Select most suitable method ---

    ViUInt16    statusByte;
    ViInt32     opcTimeout = 0;

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viEnableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE, VI_NULL));

    viCheckErr( viPrintf (io, "*OPC"));

    // wait for SRQ
    viCheckErr( viWaitOnEvent (io, VI_EVENT_SERVICE_REQ, opcTimeout, VI_NULL, VI_NULL));
    viCheckErr( viDisableEvent (io, VI_EVENT_SERVICE_REQ, VI_QUEUE));

    // clean up after SRQ
    viCheckErr( viBufWrite (io, "*CLS", 4, VI_NULL));
    viCheckErr( viReadSTB (io, &statusByte));

#else

    ViUInt16    response    = 0;
    ViInt32     counter     = 0;
    ViUInt16    intf        = VI_INTF_GPIB;
    ViInt32     opcTimeout = 0;

    checkErr( Rs_GetAttribute (instrSession, VI_NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opcTimeout), &opcTimeout));

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intf));

    /* Interface specific operations */
    switch (intf)
        {
        case VI_INTF_ASRL: /* Serial interface */

            /* Not supported */

        break;
        default: /* Default case */

            /* Check STB for errors. */
            do
                {
                error = viReadSTB (instrSession, &response);

                Rs_Delay (0.001);
                counter++;

                if (counter >= opcTimeout)
                    return VI_ERROR_TMO;

                /* Wait while STB is non zero value. Only Error/Event Queue
                   and MAV bits are taken into consideration. */
            
                } while ((response & 0x34) == 0);

            /* (check Error(Event) Queue bit in STB) */
            return ((response & 0x4) != 0)? RS_ERROR_INSTRUMENT_STATUS : error;
        }

#endif

Error:

#if 0

    viDiscardEvents (io, VI_EVENT_SERVICE_REQ, VI_QUEUE);

#endif

    return error;
}

/*****************************************************************************
 * Function: rssfc_quotedString_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes string 
 *           with apostrophes
 *****************************************************************************/
ViStatus rssfc_quotedString_WriteCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error               = VI_SUCCESS;
    ViInt32     idx                 = 0;
    ViChar      *cmd                = VI_NULL;

    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  

    /* --- If the range table for the attribute exists, pass 'command string' or
       'discreteOrMinValue' instead of attribute value. --- */

    if (sessionProperties -> attr[idx].rangeTable)
        if (sessionProperties -> attr[idx].rangeTable -> type == RS_VAL_DISCRETE)
            {
            ViUInt16    i = 0;
            ViChar      *p2string;
            ViReal64    *p2discrete;
            ViInt32     data_type = sessionProperties -> attr[idx].dataType;
            ViReal64    precision = sessionProperties -> attr[idx].comparePrecision;

            precision = (precision < DBL_MIN) ? DBL_MIN : precision;

            /* Allowed data types are only ViInt32 or ViReal64 */
            switch (data_type)
                {
                case RS_VAL_INT32:
                case RS_VAL_REAL64:
                    /* Valid data types */
                    break;
                case RS_VAL_STRING:
                case RS_VAL_BOOLEAN:
                case RS_VAL_EVENT:
                case RS_VAL_ADDR:
                case RS_VAL_SESSION:
                case RS_VAL_UNKNOWN_TYPE:
                default:
                    viCheckErr (RS_ERROR_INVALID_TYPE);
                }

            /* Go over range table values and find appropriate command string for passed
               attribute value. For floating point values use predefined attribute precision. */
            do
                {
                p2discrete = &sessionProperties -> attr[idx].rangeTable -> rangeValues[i].discreteOrMinValue;
                p2string = sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString;

                if (sessionProperties -> attr[idx].rangeTable -> rangeValues[i].cmdString == RS_RANGE_TABLE_END_STRING)
                    return RS_ERROR_INVALID_VALUE;

                if (data_type == RS_VAL_INT32)
                    if (fabs(*p2discrete - *(ViInt32 *)value) <= DBL_MIN)
                        {
                        viCheckErr (viPrintf(instrSession, "%s \'%s\'\n", cmd, p2string));
                        break;
                        }

                if (data_type == RS_VAL_REAL64)
                    if (fabs(*p2discrete - *(ViReal64 *)value) <= precision)
                        {
                        viCheckErr (viPrintf(instrSession, "%s \'%s\'\n", cmd, p2string));
                        break;
                        }

                i++;

                } while (i);

            goto Error; /* Exit the function */
            }

    /* --- Attribute data type is used for VISA I/O --- */
    switch (sessionProperties -> attr[idx].dataType)
        {
        case RS_VAL_INT32:
            viCheckErr (viPrintf(instrSession, "%s %ld\n", cmd, *(ViInt32 *)value));
            break;
        case RS_VAL_REAL64:
            viCheckErr (viPrintf(instrSession, "%s %.12f\n", cmd, *(ViReal64 *)value));
            break;
        case RS_VAL_STRING:
            viCheckErr (viPrintf(instrSession, "%s \'%s\'\n", cmd, value));
            break;
        case RS_VAL_BOOLEAN:
            viCheckErr (viPrintf(instrSession, "%s %s\n", cmd, (*(ViBoolean *)value == VI_FALSE) ? "OFF" : "ON"));
            break;
        case RS_VAL_EVENT:
            viCheckErr (viPrintf(instrSession, "%s\n", cmd));
            break;
        case RS_VAL_ADDR:
        case RS_VAL_SESSION:
        case RS_VAL_UNKNOWN_TYPE:
        default:
            viCheckErr (RS_ERROR_INVALID_TYPE);
        }

Error:

    if (cmd)
        free (cmd);

    return error;

}

/*****************************************************************************
 * Function: rssfc_hex_ReadCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rssfc_hex_ReadCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, ViInt32 bufSize, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error = VI_SUCCESS;
    ViInt32     idx             = 0;
    ViChar      *cmd            = VI_NULL,
                *buffer         = VI_NULL;
	bufSize;

    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  

   /* --- Attribute data type is used for VISA I/O --- */
    switch (sessionProperties -> attr[idx].dataType)
    {
        case RS_VAL_INT32:
            if ((sessionProperties -> attr[idx].access==RS_VAL_READ_ONLY)) 
            {
                viCheckErr (viPrintf(instrSession, "%s\n", cmd));
            }
            else
            {
                viCheckErr (viPrintf(instrSession, "%s?\n", cmd));
            }
            viCheckErr (viScanf(instrSession, "#H%X", value));
            break;
        default:
            viCheckErr (RS_ERROR_INVALID_TYPE);
    }

Error:

    if (buffer)
        free (buffer);

    if (cmd)
        free (cmd);

    return error;
};

/*****************************************************************************
 * Function: rssfc_hex_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rssfc_hex_WriteCallback(ViSession instrSession, ViConstString repCapName, ViAttr attributeId, void *value)
{
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
    ViStatus    error = VI_SUCCESS;
    ViInt32     idx                 = 0;
    ViChar      *cmd                = VI_NULL;
    
    viCheckErr (Rs_GetAttributeIndex (instrSession, attributeId, &idx));

    /* If no command available, exit the function */
    if (sessionProperties -> attr[idx].command == VI_NULL)
        return VI_SUCCESS;

    /* Check if attribute uses repeated capabilities */
    if (sessionProperties -> attr[idx].usercaps == VI_TRUE)
        {
        if (!repCapName)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);

        if (strlen (repCapName) == 0)
            viCheckErr (RS_ERROR_INVALID_REPEATED_CAPABILITY_NAME);
        }

    /* Build the command string */
    viCheckErr (Rs_BuildCommandString (instrSession, attributeId, idx, repCapName, &cmd));  

    switch (sessionProperties -> attr[idx].dataType)
    {
        case RS_VAL_INT32:
            viCheckErr (viPrintf(instrSession, "%s #H%X\n", cmd, *(ViInt32 *)value));
            break;
        default:
            viCheckErr (RS_ERROR_INVALID_TYPE);
    }
Error:

    if (cmd)
        free (cmd);

    return error;

};
/*****************************************************************************
 *- User Defined Attribute Callbacks ----------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Coerce Callbacks --------------------------------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *- Range Table Callbacks ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rssfc_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
static ViStatus rssfc_InitAttributes (ViSession instrSession)
{
    ViStatus    error           = VI_SUCCESS;
    ViInt32     p2value         = 0;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE] = "",
                revision[RS_MAX_MESSAGE_BUF_SIZE] = "",
                manufacturer[RS_MAX_MESSAGE_BUF_SIZE] = "",
                model[RS_MAX_MESSAGE_BUF_SIZE] = "",
                idQ[RS_MAX_MESSAGE_BUF_SIZE] = "",
                options_list[2*RS_MAX_MESSAGE_BUF_SIZE] = "";
	void * p2void;

    buffer[RS_MAX_MESSAGE_BUF_SIZE -1] = '\0';

    /*- Misc -*/

    if (Rs_Simulating(instrSession) == 0)
        {
        /* Instrument identification */
        viCheckErr (viQueryf (instrSession, "*IDN?\n", "%[^\r\n]", idQ));
        idQ[RS_MAX_MESSAGE_BUF_SIZE - 1] = '\0';
        checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RSSFC_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        viCheckErr (viQueryf (instrSession, "*OPT?\n", "%[^\r\n]", options_list));
        //options_list[2*RS_MAX_MESSAGE_BUF_SIZE - 1] = '\0';
        }
    else /* Simulating */
        {
        /* Instrument identification */
        strncpy (idQ, RSSFC_SIMULATION_ID_QUERY, RS_MAX_MESSAGE_BUF_SIZE - 1);
        checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RSSFC_ATTR_ID_QUERY_RESPONSE, 0, idQ));
        
        /* Option(s) string */
        strncpy (options_list, RSSFC_SIMULATION_OPT_QUERY, 2*RS_MAX_MESSAGE_BUF_SIZE - 1);
        }

    /*- Inherent Instrument Specific Attributes ----------------------------*/

    /*- User Options -*/

    /*
    RS_ATTR_RANGE_CHECK             ... Rs_SpecificDriverNew
    RS_ATTR_QUERY_INSTRUMENT_STATUS ... Rs_SpecificDriverNew
    RS_ATTR_CACHE                   ... Rs_SpecificDriverNew
    RS_ATTR_SIMULATE                ... Rs_SpecificDriverNew
    RS_ATTR_RECORD_COERCIONS        ... not supported
    RS_ATTR_INTERCHANGE_CHECK       ... not supported
    RS_ATTR_SPY                     ... not supported
    RS_ATTR_USE_SPECIFIC_SIMULATION ... not supported
    */

    /*- Class Driver Identification -*/

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_DESCRIPTION, 0, RSSFC_CLASS_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_PREFIX, 0, RSSFC_CLASS_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_VENDOR, 0, RSSFC_CLASS_DRIVER_VENDOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_REVISION, 0, RSSFC_CLASS_DRIVER_REVISION));
    p2value = RSSFC_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFC_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Identification -*/
                                                        
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSSFC_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSSFC_SPECIFIC_DRIVER_PREFIX));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSSFC_SPECIFIC_DRIVER_LOCATOR));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSSFC_SPECIFIC_DRIVER_VENDOR));

    sprintf (buffer, 
             "Driver: %s %ld.%ld.%ld, Compiler: %s %3.2f, "
             "Components: RSEngine %.2f, VISA-Spec %.2f",
             RSSFC_SPECIFIC_DRIVER_PREFIX,
             RSSFC_MAJOR_VERSION,RSSFC_MINOR_VERSION,RSSFC_MINOR_MINOR_VERSION, 
             RS_COMPILER_NAME, RS_COMPILER_VER_NUM, 
             (double)(RS_ENGINE_MAJOR_VERSION + RS_ENGINE_MINOR_VERSION)/10.0, 
             (double)Rs_ConvertVISAVer(VI_SPEC_VERSION));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, buffer));
    p2value = RSSFC_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFC_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, &p2value));

    /*- Driver Capabilities -*/
                                   
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSSFC_SUPPORTED_INSTRUMENT_MODELS));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSSFC_GROUP_CAPABILITIES));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSSFC_FUNCTION_CAPABILITIES));
    /* RS_ATTR_CHANNEL_COUNT        ... Rs_BuildRepCapTable */

    /*- Driver Setup -*/

    /* RS_ATTR_DRIVER_SETUP         ... Rs_SpecificDriverNew */

    /*- Instrument Identification -*/

    checkErr (Rs_GetAttribute (instrSession, "", (ViAttr) RSSFC_ATTR_ID_QUERY_RESPONSE, 0, RS_MAX_MESSAGE_BUF_SIZE, idQ));
    (void) sscanf (idQ, "%[^,],%[^,],%*[^,],%256[^\n]", manufacturer, model, revision);
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_INSTRUMENT_MANUFACTURER, 0, manufacturer));
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_INSTRUMENT_MODEL, 0, model));
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, revision));
    checkErr (Rs_SetAttribute (instrSession, "", (ViAttr) RS_ATTR_OPTIONS_LIST, 0, options_list));

    /*- Advanced Session Information -*/
                                  
    /*  
    RS_ATTR_IO_RESOURCE_DESCRIPTOR  ... rssfc_InitWithOptions
    RS_ATTR_LOGICAL_NAME            ... not supported
    */

    /*- Inherent Vendor Specific Attributes --------------------------------*/

    /*- Error Info -*/

    /*
    RS_ATTR_PRIMARY_ERROR           ... Rs_SetErrorInfo
    RS_ATTR_SECONDARY_ERROR         ... Rs_SetErrorInfo
    RS_ATTR_ERROR_ELABORATION       ... Rs_SetErrorInfo
    */

    /*- Session I/O -*/

    /*
    RS_ATTR_VISA_RM_SESSION         ... rssfc_RsInit
    RS_ATTR_IO_SESSION              ... Rs_UpdateIOSession
    RS_ATTR_IO_SESSION_TYPE         ... not supported
    */

    /*- Version Info -*/                     

    p2value = RSSFC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RSSFC_MINOR_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, &p2value));
    p2value = RSSFC_CLASS_SPEC_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, &p2value));
    p2value = RSSFC_CLASS_SPEC_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MAJOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MAJOR_VERSION, 0, &p2value));
    p2value = RS_ENGINE_MINOR_VERSION;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_MINOR_VERSION, 0, &p2value));
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_ENGINE_REVISION, 0, RS_ENGINE_REVISION));

    /*- Session Callbacks -*/                

    p2void = rssfc_WaitForOPCCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_CALLBACK, 0, &p2void));
    p2void = rssfc_CheckStatusCallback;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_CHECK_STATUS_CALLBACK, 0, &p2void));
    p2value = RSSFC_OPC_TIMEOUT;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_OPC_TIMEOUT, 0, &p2value));
	p2value = VI_TRUE;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));
	
    /*----------------------------------------------------------------------*/

	
Error:
    return error;
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
